<?php
/**
 * Debug info page
 *
 * @since 0.9.3
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


/**
 * Content of debug settings page
 *
 * @since 0.9.3
 */
?>
<div class="picu-tab-content js-picu-tab-content" id="picu-settings-debug">
	<table id="picu-debug-info">
		<thead>
			<tr>
				<th><?php _e( 'Setting', 'picu' ); ?></th>
				<th><?php _e( 'Value', 'picu' ); ?></th>
			</tr>
		</thead>
		<tbody>
			<tr>
				<td>Home URL</td>
				<td><?php echo home_url(); ?></td>
			<tr>
				<td>WordPress version</td>
				<td><?php echo bloginfo('version'); ?></td>
			</tr>
			<tr>
				<td>Active plugins</td>
				<td>
					<?php
					$all_plugins = get_plugins();
					$active_plugins = get_option( 'active_plugins' );
					$plugins = array();

					foreach( $active_plugins as $plugin ) {
						if ( isset( $all_plugins[$plugin] ) ) {
							array_push( $plugins, $all_plugins[$plugin] );
						}
					}

					$temp = array();
					foreach( $plugins as $plugin ) {
						$temp[] = $plugin['Name'] . ' (' . $plugin['Version'] . ')';
					}
					echo implode( ', ', $temp );
					?>
				</td>
			</tr>
			<tr>
				<td>Active theme</td>
				<td>
				<?php
					$theme = wp_get_theme();
					echo $theme->get( 'Name') . ' (' . $theme->get( 'Version') . ')<br />' . $theme->get( 'ThemeURI' );
				?>
				</td>
			</tr>
			<tr>
				<td>picu version</td>
				<td><?php echo esc_html( PICU_VERSION ); ?></td>
			</tr>
			<tr>
				<td>picu upload dir</td>
				<td><?php echo PICU_UPLOAD_DIR; ?></td>
			</tr>
			<tr>
				<td>picu base slug</td>
				<td>
				<?php
					$post_type_object = get_post_type_object( 'picu_collection' );
					echo $post_type_object->rewrite['slug'];
				?>
				</td>
			</tr>
			<tr>
				<td>PHP version</td>
				<td><?php echo esc_html( PHP_VERSION ); ?></td>
			</tr>
			<tr>
				<td>MySQL version</td>
				<?php
					global $wpdb;
				?>
				<td><?php echo esc_html( $wpdb->get_var( "SELECT VERSION() AS version" ) ); ?></td>
			</tr>
			<tr>
				<td>Server</td>
				<td><?php echo esc_html( $_SERVER[ 'SERVER_SOFTWARE' ] ); ?></td>
			</tr>
			<tr>
				<td>Operating system</td>
				<td><?php echo esc_html( PHP_OS ); ?></td>
			</tr>
			<?php
				if ( function_exists( 'get_current_user' ) ) {
			?>
			<tr>
				<td>Current PHP user</td>
				<td><?php echo esc_html( get_current_user() ); ?></td>
			</tr>
			<?php
				}
			?>
			<tr>
				<td>Safe mode</td>
				<td><?php echo (bool) ini_get( 'safe_mode' ) ? __( 'On', 'picu' ) : __( 'Off', 'picu' ); ?></td>
			</tr>
			<tr>
				<td>Maximum execution time</td>
				<td><?php echo esc_html( ini_get( 'max_execution_time' ) ) . ' ' . __( 'seconds', 'picu' ); ?></td>
			</tr>
			<tr>
				<td>Maximum number of input variables</td>
				<td><?php echo ini_get( "max_input_vars" ); ?></td>
			</tr>
			<tr>
				<td>Server time</td>
				<td><?php echo esc_html( date( 'H:i' ) ); ?></td>
			</tr>
			<tr>
				<td>Blog time</td>
				<td><?php echo date( 'H:i', current_time( 'timestamp' ) ); ?></td>
			</tr>
			<tr>
				<td>Blog language</td>
				<td><?php echo get_bloginfo( 'language' ); ?></td>
			</tr>
			<tr>
				<td>MySQL client encoding</td>
				<td><?php echo defined( 'DB_CHARSET' ) ? DB_CHARSET : ''; ?></td>
			</tr>
			<tr>
				<td>Blog charset</td>
				<td><?php echo get_bloginfo( 'charset' ); ?></td>
			</tr>
			<tr>
				<td>PHP memory limit</td>
				<td><?php echo esc_html ( ini_get( 'memory_limit' ) ); ?></td>
			</tr>
			<tr>
				<td>Maximum upload file size</td>
				<td><?php echo size_format( wp_max_upload_size() ); ?></td>
			</tr>
			<tr>
				<td>WordPress memory limit</td>
				<td><?php echo esc_html( WP_MEMORY_LIMIT ); ?></td>
			</tr>
			<tr>
				<td>WordPress maximum memory limit</td>
				<td><?php echo esc_html( WP_MAX_MEMORY_LIMIT ); ?></td>
			</tr>
			<tr>
				<td>Memory in use</td>
				<td><?php echo size_format( @memory_get_usage( TRUE ), 2 ); ?></td>
			</tr>
			<tr>
				<td>Loaded PHP extensions</td>
				<?php
					$extensions = get_loaded_extensions();
					sort( $extensions );
				?>
				<td><?php echo esc_html( implode( ', ', $extensions ) ); ?></td>
			</tr>
		</tbody>
	</table>

</div><!-- .picu-tab #picu-settings-debug"-->