<?php
/**
 * Picu edit collection
 *
 * Add our custom metabox, which replaces the default publish actions
 * Also load our custom colleciton edit screen and all its fields
 *
 * @since 0.8.0
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Register our metabox
 *
 * @since 0.3.0
 */
function picu_add_metabox() {
	add_meta_box(
		'picu-submit-metabox',
		__( 'Collection Status', 'picu' ),
		'picu_collection_metabox',
		'picu_collection',
		'side',
		'high'
	);
}

add_action( 'add_meta_boxes', 'picu_add_metabox' );


/**
 * Construct the metabox
 *
 * @since 0.3.0
 */
function picu_collection_metabox( $post ) {

	// Add a nonce field so we can check for it later
	wp_nonce_field( 'picu_collection_metabox', 'picu_collection_metabox_nonce' );

	// Get post status
	$post_status = $post->post_status;

	// Load share method
	if ( $post_status == 'delivery-draft' ) {
		$picu_collection_share_method = get_post_meta( $post->ID, '_picu_delivery_share_method', true );
	}
	else {
		$picu_collection_share_method = get_post_meta( $post->ID, '_picu_collection_share_method', true );
	}
	?>

	<div class="picu-submit-metabox-inside">
		<div class="proof-status">
		<?php
			// Default status
			$collection_status = array(
				'draft' => 10,
				'sent' => 50,
				'approved' => 100,
			);

			// Add support for custom collection status
			$collection_status = apply_filters( 'picu_collection_status', $collection_status, $post );

			// Sorty by priority
			asort( $collection_status );

			foreach( $collection_status as $key => $value ) {
				$status_output = 'picu_collection_status_' . $key;
				if ( function_exists( $status_output ) ) {
					echo $status_output( $post );
				}
			}
		?>
		</div>
	</div>

	<div class="picu-post-options"><?php
	if ( ( 'delivered' == $post_status OR 'approved' == $post_status OR 'sent' == $post_status ) AND empty( $post->post_password ) ) {
		// Show nothing
	}
	elseif ( ( 'delivered' == $post_status OR 'approved' == $post_status OR 'sent' == $post_status ) AND ! empty( $post->post_password ) ) { ?>
		<div class="picu-option-item">
			<a class="picu-option-password-protected" href="#picu-password"><span class="dashicons dashicons-lock"></span><?php _e( 'Show Password', 'picu' ); ?></a>
			<div id="picu-password" class="picu-option-content is-hidden">
				<input type="text" id="post_password" name="post_password" maxlength="20" value="<?php echo $post->post_password; ?>" disabled="disabled" />
			</div>
		</div><?php
	} else {
?>
		<div class="picu-option-item">
			<a href="#picu-password"><span class="dashicons dashicons-<?php if ( empty( $post->post_password) ) { echo 'un'; } ?>lock"></span><?php _e( 'Password Protection', 'picu' ); ?></a>
			<div id="picu-password" class="picu-option-content is-hidden">
				<div class="picu-password-wrap">
					<label for="post_password"><?php _e( 'Enter Password', 'picu' ); ?>:</label> <input type="text" id="post_password" name="post_password" maxlength="20" value="<?php echo $post->post_password; ?>" />
					<a href="#" class="picu-remove-password js-picu-remove-password is-hidden"><?php _e( 'Empty Password Field', 'picu' ); ?></a>
				</div>
				<p class="picu-hint js-picu-password-hint-send<?php if ( 'picu-copy-link' == $picu_collection_share_method ) { echo ' is-hidden'; } ?>"><?php 
				
				$options = get_option( 'picu_settings' );
				$send_password_in_email = true;
	
				if ( isset( $options['send_password'] ) AND $options['send_password'] == 'off' ) {
					$send_password_in_email = false;
				}

				$send_password_in_email = apply_filters( 'picu_send_password_in_email', $send_password_in_email );

				if ( $send_password_in_email ) { _e( 'The password will be sent to the client with the email.', 'picu' ); } else { _e( 'The password will <strong>not</strong> be sent with the email. Make sure to send it to the client separately.', 'picu' ); } 
				
				?></p>
				<p class="picu-hint js-picu-password-hint-rememeber<?php if ( empty( $picu_collection_share_method ) OR 'picu-send-email' == $picu_collection_share_method ) { echo ' is-hidden'; } ?>"><?php _e( 'Don\'t forget to sent the password to your client!', 'picu' ); ?></p>
			</div>
		</div>
	<?php } ?>
	</div><!-- .picu-post-options -->

	<div id="submitpost">
		<div id="major-publishing-actions">
		<?php
			if ( 'delivered' == get_post_status() ) {
		?>
				<a class="button js-picu-edit" href="<?php print wp_nonce_url( admin_url( "post.php?post=" . $post->ID . "&action=edit" ), 'picu_collection_reopen_' . $post->ID, 'reopen' ); ?>"><?php _e( 'Edit Delivery', 'picu' ); ?></a>

				<div class="picu-modal picu-warning is-hidden" id="js-picu-edit">
					<div class="picu-modal-inner">
						<div class="picu-modal-content">
							<h3><?php _e( 'Caution!', 'picu' ); ?></h3><p><?php _e ( 'You already delivered this collection to your client.', 'picu' ); ?></p><p><strong><?php _e ( 'Are you sure you want to make changes?', 'picu' ); ?></strong></p>
							<div class="picu-modal-actions">
								<a class="button button-primary" href="<?php print wp_nonce_url( admin_url( "post.php?post=" . $post->ID . "&action=edit" ), 'picu_collection_reopen_' . $post->ID, 'reopen' ); ?>"><?php _e( 'Yes, I am sure', 'picu' ); ?></a> <a class="button picu-cancel-modal js-picu-cancel-modal" href=""><?php _e( 'Cancel', 'picu' ); ?></a>
							</div>
						</div>
					</div>
				</div>
		<?php
			}
			elseif ( 'sent' == get_post_status() ) {
		?>
				<a class="button js-picu-edit" href="<?php print wp_nonce_url( admin_url( "post.php?post=" . $post->ID . "&action=edit" ), 'picu_collection_reopen_' . $post->ID, 'reopen' ); ?>"><?php _e( 'Edit Collection', 'picu' ); ?></a>
				<a class="button" href="<?php echo wp_nonce_url( admin_url( 'post.php?picu_duplicate_collection=' . $post->ID ), 'picu_duplicate_collection', 'picu_duplication_nonce' ); ?>"><?php _e( 'Duplicate', 'picu' ); ?></a>

				<div class="picu-modal picu-warning is-hidden" id="js-picu-edit">
					<div class="picu-modal-inner">
						<div class="picu-modal-content">
							<h3><?php _e( 'Caution!', 'picu' ); ?></h3><p><?php _e ( 'You already sent this collection to the client.', 'picu' ); ?></p><p><strong><?php _e ( 'Are you sure you want to make changes?', 'picu' ); ?></strong></p>
							<div class="picu-modal-actions">
								<a class="button button-primary" href="<?php print wp_nonce_url( admin_url( "post.php?post=" . $post->ID . "&action=edit" ), 'picu_collection_reopen_' . $post->ID, 'reopen' ); ?>"><?php _e( 'Yes, I am sure', 'picu' ); ?></a> <a class="button picu-cancel-modal js-picu-cancel-modal" href=""><?php _e( 'Cancel', 'picu' ); ?></a>
							</div>
						</div>
					</div>
				</div>

		<?php
			}
			// The wrapping #submitbox is needed for the submit buttons to work
			elseif ( 'approved' != get_post_status() ) {
		?>
				<span class="picu-save-button-wrap">
				<?php
				// Adds a submit button to save the collection without sending
				submit_button( __( 'Save', 'picu' ), 'save-draft large', 'save', false, array( 'id' => 'save-post' ) );
				?>
				<span class="spinner"></span>
				</span>
				<?php
				// Filter initially seleced share method, default is send via email
				$default_share_method = apply_filters( 'picu_default_share_method_is_email', true );

				if ( ( ! empty( $picu_collection_share_method ) AND 'picu-send-email' == $picu_collection_share_method ) OR ( empty( $picu_collection_share_method ) AND $default_share_method == true ) ) {
					$button_text = __( 'Send to Client', 'picu' );
				}
				else {
					$button_text = __( 'Publish', 'picu' );
				}

				// Adds a submit button to publish and send the collection to the client
				submit_button( $button_text, 'primary large', 'picu_sendmail', false, array( 'id' => 'publish' ) );

			}
			else {
		?>
				<a class="button js-picu-edit" href="<?php print wp_nonce_url( admin_url( "post.php?post=" . $post->ID . "&action=edit" ), 'picu_collection_reopen_' . $post->ID, 'reopen' ); ?>"><?php _e( 'Reopen Collection', 'picu' ); ?></a>
				<?php if ( picu_get_selection_count( $post->ID ) > 0 ) { ?>
				<a class="button js-picu-duplicate" data-id="<?php echo $post->ID; ?>" href="<?php echo wp_nonce_url( admin_url( 'post.php?picu_duplicate_collection=' . $post->ID ), 'picu_duplicate_collection', 'picu_duplication_nonce' ); ?>"><?php _e( 'Duplicate', 'picu' ); ?>&hellip;</a>
				<?php } else { ?>
					<a class="button" href="<?php echo wp_nonce_url( admin_url( 'post.php?picu_duplicate_collection=' . $post->ID ), 'picu_duplicate_collection', 'picu_duplication_nonce' ); ?>"><?php _e( 'Duplicate', 'picu' ); ?></a>
		<?php 
			}
		?>
				<div class="picu-modal picu-warning is-hidden" id="js-picu-edit">
					<div class="picu-modal-inner">
						<div class="picu-modal-content">
							<h3><?php _e( 'Caution!', 'picu' ); ?></h3><p><?php _e ( 'This collection has already been approved.', 'picu' ); ?></p><p><strong><?php 
							if ( picu_is_multi_collection( $post->ID ) ) {
								_e ( 'Are you sure you want to reopen it for all clients?', 'picu' );
							}
							else {
								_e ( 'Are you sure you want to reopen it?', 'picu' );
							} ?></strong></p>
							<div class="picu-modal-actions">
								<a class="button button-primary" href="<?php print wp_nonce_url( admin_url( "post.php?post=" . $post->ID . "&action=edit" ), 'picu_collection_reopen_' . $post->ID, 'reopen' ); ?>"><?php _e( 'Yes, I am sure', 'picu' ); ?></a> <a class="button picu-cancel-modal js-picu-cancel-modal" href="#"><?php _e( 'Cancel', 'picu' ); ?></a>
							</div>
						</div>
					</div>
				</div>

			<?php echo picu_get_duplication_modal( $post->ID );
		}
			do_action( 'picu_after_major_publishing_actions', $post ); ?>

			<a class="picu-delete" href="<?php echo get_delete_post_link( $post->ID ); ?>"><?php _e( 'Move to Trash' ); ?></a>
		</div>
	</div>
<?php
}


/**
 * Generate output for draft collection status
 * 
 * @since 1.5.0
 */
function picu_collection_status_draft( $post ) {
	$post_status = $post->post_status;

	$status = '';
	if ( 'auto-draft' == $post_status OR 'draft' == $post_status OR 'publish' == $post_status ) {
		$status .= '<span class="status create"><span class="status-label">' . __( 'Create collection', 'picu' ) . '</span>';

		$num = picu_get_collection_image_num( $post->ID );
		if ( $num > 0 ) {
			$status .= '<span class="status__meta">' . $num . ' ' . _n( 'image', 'images', $num, 'picu' ) . '</span>';
		}

		$status .= '</span>';
	}
	return $status;
}


/**
 * Generate output for sent collection status
 * 
 * @since 1.5.0
 */
function picu_collection_status_sent( $post ) {
	$post_status = $post->post_status;
	$sent_time = picu_get_collection_history_event_time( $post->ID, 'sent' );
	
	$status = '<span class="status';
	if ( ( 'sent' == $post_status OR 'approved' == $post_status OR 'delivery-draft' == $post_status OR 'delivered' == $post_status ) AND ! empty( $sent_time ) ) {
		$status .= ' done';
	}
	$status .= '"><span class="status-label">' . __( 'Sent', 'picu' ) . '</span>';
	if ( $sent_time != false ) {
		$status .= '<span class="status__meta">';
		$num = picu_get_collection_image_num( $post->ID );
		if ( $num > 0 ) {
			$status .= ' ' . $num . ' ' . _n( 'image', 'images', $num, 'picu' ) . '<br />';
		}
		$status .= date( get_option( 'date_format' ), $sent_time ) . ', ' . date( get_option( 'time_format' ), $sent_time );
		$status .= '</span>';
	}
	$status .= '</span>';

	return $status;
}


/**
 * Generate output for approved collection status
 * 
 * @since 1.5.0
 */
function picu_collection_status_approved( $post ) {
	$post_status = $post->post_status;
	$approval_time = picu_get_collection_history_event_time( $post->ID, 'approved' );

	$status = '<span class="status';
	if ( 'sent' == $post_status ) {
		$status .= ' waiting';
	}
	elseif ( 'approved' == $post_status ) {
		$status .= ' active';
	}
	elseif ( ( 'delivery-draft' == $post_status OR 'delivered' == $post_status ) AND ! empty( $approval_time ) ) {
		$status .= ' done';
	}
	$status .= '"><span class="status-label">';
	if ( 'sent' == $post_status ) {
		$status .= __( 'Waiting for approval', 'picu' );
	} else { 
		$status .= __( 'Approved', 'picu' );
	}
	$status .= '</span>';
	if ( $approval_time != false ) { 
		$status .= '<span class="status__meta">' . date( get_option( 'date_format' ), $approval_time ) . ', ' . date( get_option( 'time_format' ), $approval_time ) . '</span>';
	}
	$status .= '</span>';

	return $status;
}



/**
 * Construct main edit screen
 *
 * @since 0.8.0
 */
function picu_main_edit_screen( $post ) {

	if ( $post->post_type != 'picu_collection' ) {
		return;
	}

	$post_status = $post->post_status;

	// Add our default post status
	$post_statuses = array(
		'approved' => 'picu_display_approved_view',
		'sent' => 'picu_display_approved_view',
	);

	// Make it possible to add custom edit views per post status via filter
	$post_statuses = apply_filters( 'picu_edit_screen_post_status', $post_statuses );

	if ( array_key_exists( $post_status, $post_statuses ) ) {
		// Execute callback function to display edit view by post status
		$post_statuses[$post_status]( $post );
	}
	/*
	 * Display draft view by default
	 */
	else {

		picu_display_draft_view( $post );

		/*
		 * Include collection options
		 */
		$step = 2;

		// Pro modules use this filter to add their options
		$picu_collection_options = apply_filters( 'picu_collection_options', false );

		if ( is_array( $picu_collection_options ) AND 0 < count( $picu_collection_options ) ) {

			$picu_collection_options_output = '';

			// Wrap options in divs
			foreach ( $picu_collection_options as $key => $option ) {
				$picu_collection_options_output .= '<div class="picu-option-set" id="' . $key . '">' . $option . '</div><!-- .picu-option-set#' . $key . ' -->';
			}

			echo '<div class="picu-collection-options"><h2><span class="stepcounter">' . $step . '</span>' . __( 'Collection Options', 'picu' ) . '</h2>' . $picu_collection_options_output . '</div><!-- .picu-collection-options -->';

			// Add one to the step number
			$step++;
		}

		picu_display_pro_hint();

		picu_display_share_options_form( $post, $step );

	}
}

add_action( 'edit_form_after_title', 'picu_main_edit_screen' );


/**
 * Display the share options on the collection edit screen
 * 
 * @since 1.5.0
 */
function picu_display_share_options_form( $post, $step = 2, $disabled = false ) {

	$picu_collection_share_method = get_post_meta( $post->ID, '_picu_collection_share_method', true );

	// Filter initially seleced share method, default is send via email
	$default_share_method = apply_filters( 'picu_default_share_method_is_email', true );

	// If share method has been saved or is empty and true for default, use send email...
	if ( ( ! empty( $picu_collection_share_method ) AND 'picu-send-email' == $picu_collection_share_method ) OR ( empty( $picu_collection_share_method ) AND $default_share_method == true ) ) {
		$use_email_method = true;
	}
	else {
		$use_email_method = false;
	}

	// Check type, either delivery or regular collection
	$type = ( 'delivery-draft' == $post->post_status ) ? 'delivery' : 'collection';

	${'picu_' . $type . '_email_address'} = get_post_meta( $post->ID, '_picu_' . $type . '_email_address', true );
	${'picu_' . $type . '_description'} = get_post_meta( $post->ID, '_picu_' . $type . '_description', true );

	// Try to prefill email address from previous step
	if ( $type == 'delivery' AND empty( $picu_delivery_email_address ) ) {
		$temp = get_post_meta( $post->ID, '_picu_collection_email_address', true );

		if ( ! empty( $temp ) ) {
			$picu_delivery_email_address = $temp;
		}
	}

	ob_start();
?>
<div class="picu-share-options">
<h2><?php if ( ! $disabled ) { ?><span class="stepcounter"><?php echo ( $step ) ?: '2'; ?></span> <?php } ?><?php _e( 'Share Options', 'picu' ); ?></h2>
	<ul class="picu-share-select<?php if ( ! $disabled ) { echo ' js-picu-share-select'; } ?>">
		<li><a<?php if ( $use_email_method ) { echo ' class="active"'; } ?> href="#picu-send-email"><?php _e( 'Send via email', 'picu' ); ?></a></li>
		<li><a<?php if ( $use_email_method == false ) { echo ' class="active"'; } ?> href="#picu-copy-link"><?php _e( 'Copy link &amp; send manually', 'picu' ); ?></a></li>
	</ul>
	<?php if ( ! $disabled ) { ?>
	<input type="hidden" class="js-picu_collection_share_method" name="picu_collection_share_method" value="<?php if ( $use_email_method == false ) { echo 'picu-copy-link'; } else { echo 'picu-send-email'; } ?>" />
	<?php } ?>
	<div class="picu-share-option<?php if ( $use_email_method ) { echo ' is-active'; } ?>" id="picu-send-email">
		<?php $user_name = get_the_author_meta( 'display_name', $post->post_author ); ?>
		<p><label for="picu-<?php echo $type; ?>-email-address" title="<?php _e( 'Enter your clients email address', 'picu' ); ?>"><?php _e( 'Client Email', 'picu' ); ?>:</label>
			<span class="picu-share-option-email-field"><?php
					$addresses = array_filter( explode( ', ', ${'picu_' . $type . '_email_address'} ) );
					foreach ( $addresses as $address ) {
						echo '<span class="email-address">' . $address . '<span class="delete-email-address"></span></span>';
					}
				?><textarea id="picumultiemailfield" rows="1" spellcheck="false" autocomplete="false" autocapitalize="off" autocorrect="off" tabindex="1" aria-autocomplete="list" aria-haspopup="false" aria-expanded="false" autocomplete="off"></textarea></span>
			<input type="text" id="picu-<?php echo $type; ?>-email-address" name="picu_<?php echo $type; ?>_email_address" value="<?php echo ${'picu_' . $type . '_email_address'}; ?>" <?php if ( $disabled ) { echo 'disabled="disabled"'; } ?> autocomplete="off" />
		<?php if ( ! picu_pro_is_active() ) { ?>
		<span class="picu-multi-client-pro-hint hidden"><span class="picu-multi-client-pro-hint__icon">🚀</span> <span class="picu-multi-client-pro-hint__content"><?php
			/* translators: Opening and closing link tags */
			echo sprintf( __( 'Send your collection to multiple clients with %spicu Pro%s.', 'picu' ), '<a href="https://picu.io/pro/?utm_source=picu_plugin&utm_medium=mutli-client-hint" target="_blank">', '</a>' );
		?></span></span></p>
		<?php } ?>
		<?php if ( picu_pro_is_active() AND ! picu_is_multi_client_compatible() ) { ?>
		<span class="picu-multi-client-pro-hint hidden"><span class="picu-multi-client-pro-hint__icon">⚠️</span> <span class="picu-multi-client-pro-hint__content"><?php _e( 'You need to update your <strong>picu Pro modules</strong> to use multi-client collections.', 'picu' ); ?> <a href="https://picu.io/docs/faq/#multi-client-collections" target="_blank"><?php _e( 'More info', 'picu' ); ?></a></span></span></p>
		<?php } ?>

		<p><label for="picu-<?php echo $type; ?>-description" title="<?php _e( 'The description will be sent to your client via email.', 'picu' ); ?>"><?php _e( 'Message', 'picu' ); ?>:</label>
			<textarea class="js-picu-collection-description" id="picu-<?php echo $type; ?>-description" name="picu_<?php echo $type; ?>_description" cols="60" rows="13"<?php if ( $disabled ) { echo ' disabled="disabled"'; } ?>><?php if ( esc_attr( ${'picu_' . $type . '_description'} ) ) {
					echo esc_attr( ${'picu_' . $type . '_description'} );
				} elseif ( 'delivery-draft' == $post->post_status ) {
					$mail_message = sprintf( __( 'Dear Client,&#10;&#10;Your images are ready!&#10;&#10;You can download them by following the link below.&#10;&#10;Sincerely,&#10;%s', 'picu' ), $user_name );
					$mail_message = apply_filters( 'picu_delivery_client_mail_message', $mail_message, $user_name );
					echo $mail_message;
				} elseif ( 'auto-draft' == $post->post_status ) {
					$mail_message = sprintf( __( 'Dear Client,&#10;&#10;Please select the photos you like and send your selection back to us. We will start post-production as soon as we have your approval.&#10;&#10;Sincerely,&#10;%s', 'picu' ), $user_name );
					$mail_message = apply_filters( 'picu_client_mail_message', $mail_message, $user_name );
					echo $mail_message;
				}
			?></textarea></p>
			<?php
				do_action( 'picu_after_collection_description' );
			?>
	</div><!-- .picu-share-option #picu-send-email -->
	<div class="picu-share-option<?php if ( $use_email_method == false ) { echo ' is-active'; } ?>" id="picu-copy-link">
			<p><label for="picu-collection-link"><?php _e( 'Copy URL', 'picu' ); ?></label><input type="text" id="picu-collection-link" name="picu-collection-link" value="<?php echo get_draft_permalink( $post->ID ); ?>" <?php if ( $disabled ) { echo 'disabled="disabled"'; } ?>/></p>
			<?php
			// Display multi-client hint for Pro users
			if ( picu_pro_is_active() AND 'delivery-draft' != $post->post_status ) {
			?>
			<p class="picu-hint"><?php _e( '<strong>Please note:</strong> If you want to send the collection to more than one client, use "Send via email". <a class="picu-help" href="https://picu.io/docs/faq/#multi-client-collections">More info</a>', 'picu' ); ?></p>
			<?php } else { ?>
			<p class="picu-hint"><?php _e( '<strong>Please note:</strong> picu will <strong>NOT</strong> send an email. Make sure to copy and send the link to your client manually.', 'picu' ); ?></p>
			<?php } ?>
	</div><!-- .picu-share-option #picu-copy-link -->
</div><!-- .picu-share-options -->
<?php
	echo ob_get_clean();
}



/**
 * Display the draft view on the collection edit screen
 * 
 * @since 1.5.0
 */
function picu_display_draft_view( $post ) {

	// Load the IDs of all uploaded images into an array
	$gallery_data = get_post_meta( $post->ID, '_picu_collection_gallery_ids', true );
	if ( ! empty( $gallery_data ) ) {
		$gallery_image_ids = explode( ',', $gallery_data );
		$gallery_image_count = count( $gallery_image_ids );
	}
	else {
		$gallery_image_ids = '';
		$gallery_image_count = 0;
	}

	$gallery_class = '';
	if ( ! empty( $gallery_data ) ) {
		$gallery_class = ' picu-gallery-has-images';
	}
	if ( 10 < $gallery_image_count ) {
		$gallery_class .= ' is-collapsible js-collapsed';
	}

	ob_start();
?>
	<div class="postbox picu-postbox <?php echo $gallery_class; ?>">
		<div class="picu-postbox-inner">
			<?php
				$picu_section_header_1 = __( 'Upload Images', 'picu' );
				$picu_section_header_1 = apply_filters( 'picu_section_header_1', $picu_section_header_1 );
			?>
			<h2><span class="stepcounter">1</span> <?php echo $picu_section_header_1; ?></h2>
			<div class="picu-gallery-thumbnails">
			<?php

			// Define the drag & drop zone for our uploader
			if ( ! empty( $gallery_image_ids ) ) {

				// Loop through all uploaded images
				foreach ( $gallery_image_ids as $gallery_image_id ) {

					// Load filename
					$img_name = wp_get_attachment_image_src( $gallery_image_id, 'full' );

					// Define the attributes to output with our thumbnails
					$attr = array(
						'title' => basename( $img_name[0] ),
						'draggable' => 'false'
					);

					// Construct the image markup
					echo '<figure>';
					echo '<div class="picu-gallery-thumbnail-box">';
					echo '<div class="picu-gallery-thumbnail-box-inner">';
					echo wp_get_attachment_image( $gallery_image_id, 'picu-small', 0, $attr );
					echo '</div></div></figure>';
				}
				
			}
			?>
			</div><!-- .picu-gallery-thumbnails -->
			<div class="toggle-picu-gallery-height">
				<a href="#" class="js-toggle-picu-gallery-height"><span class="show"><?php _e( 'Show all images', 'picu' ); ?></span><span class="hide"><?php _e( 'Hide images', 'picu' ); ?></span></a>
			</div>

			<?php
			$picu_before_upload = '';
			$picu_before_upload = apply_filters( 'picu_before_upload', $picu_before_upload );
			echo $picu_before_upload;

			?>
			<div class="picu-gallery-uploader">
				<input type="text" id="picu-gallery-ids" name="picu_gallery_ids" class="hidden" value="<?php echo $gallery_data; ?>">
				<?php wp_nonce_field( 'picu_gallery_ids', 'picu_gallery_ids_nonce' ); ?>
				<p class="picu-drag-info"><?php _e( 'Drag and drop your images here or click the button to upload', 'picu' ); ?></p>
				<p><a class="button picu-upload-image-button" href="#"><?php _e( 'Upload / Edit Images', 'picu' ); ?></a></p>
				<p class="picu-max-file-size"><?php echo __( 'Maximum upload size', 'picu' ) . ': ' . size_format( wp_max_upload_size() ); ?> <a class="picu-help" href="https://picu.io/faq#maximum-upload-size" target="_blank"><?php _e( 'Help', 'picu' ); ?></a></p>
			</div><!-- .picu-gallery-uploader -->
		</div><!-- .picu-postbox-inner -->
	</div><!-- .postbox.picu-postbox -->
<?php
	echo ob_get_clean();
}



/**
 * Display the approved view on the collection edit screen
 * 
 * @since 1.5.0
 * 
 * @param object $post The post object
 * @param bool $collapsible Wether to make the postbox collapsible
 */
function picu_display_approved_view( $post, $collapsible = false ) {

	// Gather collection infos
	$collection_status = $post->post_status;
	$picu_collection_share_method = get_post_meta( $post->id, '_picu_collection_share_method', true );
	$picu_collection_gallery_ids = get_post_meta( $post->ID, '_picu_collection_gallery_ids', true );
	$gallery_image_ids = explode( ',', $picu_collection_gallery_ids );
	$gallery_image_count = count( $gallery_image_ids );

	// Prepare selection data
	$selection_image_count = 0;
	$selection_image_ids = [];
	$selection_data = get_post_meta( $post->ID, '_picu_collection_selection', true );
	if ( ! empty( $selection_data ) ) {
		$selection_image_ids = $selection_data['selection'];
		$selection_image_count = ( is_array( $selection_data['selection'] ) ) ? count( $selection_data['selection'] ) : 0;
	}
	$img_filenames = picu_get_approved_filenames( $post->ID );

	// Get data for mutli client collections and prepare selection data
	$picu_collection_hashes = get_post_meta( $post->ID, '_picu_collection_hashes', true );
	$multi_selection_image_ids = [];
	$picu_multi_selections = [];
	$picu_multi_selected_by_all = [];
	$picu_multi_selected_by_no_one = [];
	$picu_multi_selected_by_all_count = 0; // To determine how many clients have made a selection
	$picu_multi_approval_messages = [];
	$picu_multi_approval_fields = [];
	$i = 0;

	// Prepare image votes
	$image_votes = [];
	foreach( $gallery_image_ids as $id ) {
		$image_votes['image_' . $id] = 0;
	}

	// Calculate different selection for multi-client collections
	if ( ! empty( $picu_collection_hashes ) ) {

		foreach( $picu_collection_hashes as $key => $hash ) {

			// Fill individual selections
			$selection = get_post_meta( $post->ID, '_picu_collection_selection_' . $key, true );

			if ( ! empty( $selection ) ) {
				if ( isset( $selection['selection'] ) ) {
					$picu_multi_selections[$key] = $selection['selection'];
				}

				// Fill filenames
				if ( ! empty( $selection['selection'] ) ) {
					$multi_selection_image_ids = array_merge( $multi_selection_image_ids, $selection['selection'] );
				}

				// Fill selected by all
				if ( $i === 0 ) {
					// Set the selection for the first time
					if ( isset( $selection['selection'] ) AND is_array( $selection['selection'] ) ) {
						$picu_multi_selected_by_all = $selection['selection'];
					}
					$picu_multi_selected_by_all_count++;
				}
				else {
					if ( isset( $selection['selection'] ) AND is_array( $selection['selection'] ) ) {
						$picu_multi_selected_by_all = array_intersect( $picu_multi_selected_by_all, $selection['selection'] );
						$picu_multi_selected_by_all_count++;
					}
				}
				$i++;

				// Gather images selected by no one 
				if ( ! empty( $selection['selection'] ) ) {
					$picu_multi_selected_by_no_one = array_merge( $picu_multi_selected_by_no_one, $selection['selection'] );
				}
				
				// Fill multi client approval messages
				if ( isset( $selection['approval_message'] ) ) {
					$picu_multi_approval_messages[$key] = $selection['approval_message'];;
				}

				// Fill multi client approval fields
				if ( isset( $selection['approval_fields'] ) ) {
					$picu_multi_approval_fields[$key] = $selection['approval_fields'];;
				}

				// Count votes
				if ( ! empty( $selection['selection'] ) ) {
					foreach( $selection['selection'] as $value ) {
						$image_votes['image_' . $value]++;
					}
				}
			}
		}


		// Get all the at least once selected images
		$multi_selection_image_ids = array_unique( $multi_selection_image_ids );
		$selection_image_ids = $multi_selection_image_ids;

		// Update selection count (defaults to selected at least once)
		$selection_image_count = count( $multi_selection_image_ids );

		// Get images that are not selected by anyone
		$picu_multi_selected_by_no_one = array_diff( $gallery_image_ids, array_unique( $picu_multi_selected_by_no_one ) );

		// Sort by number of votes
		arsort( $image_votes );
	}

	ob_start();

	if ( picu_get_collection_history_event_time( $post->ID, 'approved' ) AND ( $collection_status == 'delivery-draft' OR $collection_status == 'delivered' ) ) { ?>
	<input type="checkbox" class="picu-toggle-approved-view-toggle" id="picu-toggle-approved-view" autocomplete="off" />
	<?php } ?>

	<div class="postbox picu-postbox<?php if ( $collection_status == 'delivery-draft' OR $collection_status == 'delivered' ) { echo ' picu-postbox-approved'; } ?>">
		<header class="picu-postbox-header">
			<h2><?php if ( $collection_status == 'sent' ) { _e( 'Selection Summary, so far...', 'picu' ); } else { _e( 'Approval Summary', 'picu' ); } ?>
			<?php if ( $collection_status == 'delivery-draft' OR $collection_status == 'delivered' ) { ?>
			<div class="picu-toggle-approved-view">
				<label class="picu-toggle-show-approved-view" for="picu-toggle-approved-view"><?php _e( 'Show Details', 'picu' ); ?></label>
				<label class="picu-toggle-hide-approved-view" for="picu-toggle-approved-view"><?php _e( 'Hide Details', 'picu' ); ?></label>
			</div>
			<?php } ?>
			</h2>
			<a class="button button-primary<?php if ( ( $collection_status != 'approved' AND $collection_status != 'delivery-draft' AND $collection_status != 'delivered' ) OR empty( $img_filenames ) ) { echo ' disabled'; } ?>" role="button" tabindex="0" href="<?php if ( $collection_status == 'approved' OR $collection_status == 'delivery-draft' OR $collection_status == 'delivered' ) { echo admin_url( 'post.php?post=' . $post->ID . '&action=edit&picu-download=picu-proof-file' ); } else { echo '#'; } ?>"><?php _e( 'Download Proof', 'picu' ); ?> (.txt)</a>
		</header>

		<div class="picu-postbox-inner">
			<div class="picu-approval-summary">
				<div class="recipient-wrap">
			<?php 
				// Multi-Client collection
				if ( ! empty( $picu_collection_hashes ) ) { ?>
			<?php
				// Iterate through multi clients
				foreach( $picu_collection_hashes as $key => $hash ) {

					$status = $hash['status'];
					$email = $hash['email'];
					$date = date( get_option( 'date_format' ) . ', ' . get_option( 'time_format' ), $hash['time'] );

					$selection = get_post_meta( $post->ID, '_picu_collection_selection_' . $key, true );

					$selection_count = '';
					if ( ! empty( $selection['selection'] ) ) {
						$selection_count = count( $selection['selection'] );
					}

					$approval_fields = [];
					if ( ! empty( $selection['approval_fields'] ) ) {
						$approval_fields = $selection['approval_fields'];
					}

					switch ( $status ) {
						case 'approved':
							$status_label = __( 'Approved', 'picu' );
							break;
						default:
							$status = 'waiting';
							$status_label = __( 'Waiting', 'picu' );
					}
				?>
					<div class="recipient recipient--<?php echo $status; ?>">
						<div class="recipient__inner">
							<span class="recipient__status-wrap">
								<span class="recipient__status" title="<?php echo $date; ?>"><?php echo $status_label; ?></span>
								<?php if ( $selection_count ) { ?>
								<span class="reciepient__selection-count"><?php echo $selection_count; ?></span>
								<?php } ?>
							</span>
							<span class="recipient__email"><?php echo $email; ?></span>
							<?php
							if ( get_post_status( $post->ID ) == 'approved' OR get_post_status( $post->ID ) == 'sent' ) { ?>
							<nav class="recipient__actions">
								<ul>
									<li><a href="<?php echo esc_url( add_query_arg( 'ident', $key, get_the_permalink() ) ); ?>"><?php _e( 'View', 'picu' ); ?></a></li>
									<?php
									if ( $status == 'approved' ) {
										// Reopen collection for this client
									?>
										<li><a href="<?php print wp_nonce_url( admin_url( 'post.php?post=' . $post->ID . '&action=edit&client=' . $key ), 'picu_collection_reopen_' . $post->ID, 'reopen' ); ?>"><?php _e( 'Reopen', 'picu' ); ?></a></li>
									<?php
									}
									if ( $status == 'waiting' ) {
										// TODO: Send reminder to client
									}
										// TODO: Delete client from this collection
									?>
								</ul>
							</nav>
							<?php }
								// Legacy: Add the old approval message to the approval fields array
								if ( ! empty( $selection['approval_message'] ) AND $status == 'approved' ) {
									$approval_fields['picu_approval_message'] = [
										'label' => __( 'Message', 'picu' ),
										'value' => $selection['approval_message'],
									];
								}
								// Display approval fields
								picu_display_approval_fields( $approval_fields, $date );
							?>
						</div><!-- .recipient__inner -->
					</div>
				<?php }
				}

			// Single-Client collection
				else {
					$status = $collection_status;
					$email = $client_email = get_post_meta( $post->ID, '_picu_collection_email_address', true );
					$date = date( get_option( 'date_format' ) . ', ' . get_option( 'time_format' ), picu_get_collection_history_event_time( $post->ID, 'approved' ) );

					$selection = get_post_meta( $post->ID, '_picu_collection_selection', true );

					$selection_count = '';
					if ( ! empty( $selection['selection'] ) ) {
						$selection_count = count( $selection['selection'] );
					}

					$approval_fields = [];
					if ( ! empty( $selection['approval_fields'] ) ) {
						$approval_fields = $selection['approval_fields'];
					}

					switch ( $status ) {
						case 'approved':
							$status_label = 'Approved';
							break;
						default:
							$status = 'waiting';
							$status_label = 'Waiting';
					}
				?>
					<div class="recipient recipient--<?php echo $status; ?>">
						<div class="recipient__inner">
							<span class="recipient__status-wrap">
								<span class="recipient__status" title="<?php echo $date; ?>"><?php echo $status_label; ?></span>
								<?php if ( $selection_count ) { ?>
								<span class="reciepient__selection-count"><?php echo $selection_count; ?></span>
								<?php } ?>
							</span>
							<?php if ( ! empty( $email ) ) { ?>
							<span class="recipient__email"><?php echo $email; ?></span>
							<?php } ?>
							<?php
							if ( get_post_status( $post->ID ) == 'approved' OR get_post_status( $post->ID ) == 'sent' ) { ?>
							<nav class="recipient__actions">
								<ul>
									<li><a href="<?php echo get_the_permalink(); ?>"><?php _e( 'View', 'picu' ); ?></a></li>
									<?php
									if ( $status == 'approved' ) {
										// Reopen collection for this client
									?>
										<li><a href="<?php print wp_nonce_url( admin_url( 'post.php?post=' . $post->ID . '&action=edit' ), 'picu_collection_reopen_' . $post->ID, 'reopen' ); ?>"><?php _e( 'Reopen', 'picu' ); ?></a></li>
									<?php
									}
									if ( $status == 'waiting' ) {
										// TODO: Send reminder to client
									}
										// TODO: Delete client from this collection
									?>
								</ul>
							</nav>
						<?php }
							// Legacy: Add the old approval message to the approval fields array
							if ( ! empty( $selection['approval_message'] ) AND $status == 'approved' ) {
								$approval_fields['picu_approval_message'] = [
									'label' => __( 'Message', 'picu' ),
									'value' => $selection['approval_message'],
								];
							}
							// Display approval fields
							picu_display_approval_fields( $approval_fields, $date );
						?>
						</div><!-- .recipient__inner -->
					</div>
			<?php 
				}
			?>
				</div><!-- .recipients-wrap -->

				<?php do_action( 'picu_after_recipients' ); ?>

			</div><!-- .picu-approval-summary -->
			<div class="picu-toolbar">
				<div class="picu-filter">
					<label for="picu-table-filter-show"><?php _e( 'Show', 'picu' ); ?>:</label>
						<?php
							// Switch the option titles, depending on collection status
							if ( $selection_image_count > 0 ) {
								if ( $collection_status == 'sent' ) {
									$filter_options = array(
										'selected' => __( 'Selected', 'picu' ),
									);
								}
								else {
									$filter_options = array(
										'selected' => __( 'Approved', 'picu' ),
									);
								}
							}

							// Add multi client filter options
							if ( ! empty( $picu_collection_hashes ) ) {
								if ( $selection_image_count > 0 ) {
									$filter_options['selected-by-all'] = __( 'Selected by all', 'picu' );
									$filter_options['selected'] = __( 'Selected at least once', 'picu' );

									// Add filter by individual client
									foreach( $picu_collection_hashes as $key => $hash ) {
										$filter_options[$key] = 'Selected by ' . $hash['email'];
									}
								}
							}

							// Add 'not selected' filter
							$filter_options['not-selected'] = __( 'Not selected', 'picu' );
							// Add 'all' filter
							$filter_options['all'] = __( 'All', 'picu' );

							// Add filter for custom options
							$filter_options = apply_filters( 'picu_table_filter_options', $filter_options, $post );

						?>
						<select id="picu-table-filter-show" autocomplete="off">
						<?php
							foreach( $filter_options as $key => $option ) {
								echo '<option value="' . $key . '">' . $option . '</option>';
							}
						?>
					</select>
				</div><!-- .picu-filter -->

				<div class="picu-copy">
					<label for="picu-copy-filenames"><?php _e( 'Copy Filenames', 'picu' ); ?>:</label>
					<input id="picu-copy-filenames" type="text" value="<?php echo $img_filenames; ?>" />
					<span class="button button-primary picu-copy-to-clipboard<?php if ( empty( $img_filenames ) ) { echo ' disabled'; } ?>" role="button" tabindex="0" data-clipboard-text="<?php echo $img_filenames; ?>"><?php printf( _n( 'Copy %s Filename', 'Copy Filenames %s', $selection_image_count, 'picu' ), '<span class="filename-count">(' .number_format_i18n( $selection_image_count ) . ')</span>' ); ?></span>
				</div><!-- .picu-copy -->
			</div><!-- .picu-toolbar -->

			<?php
				/**
				 * Table View
				 */

				// Prepare column array
				$picu_overview_table_columns = array(
					'thumbnail' => __( 'Thumbnail', 'picu' ),
					'file' => __( 'File', 'picu' ),
					'approved' => __( 'Approved', 'picu' ),
				);

				if ( $collection_status == 'sent' ) {
					$picu_overview_table_columns['approved'] = __( 'Selected', 'picu' );
				}

				// Add a filter
				$picu_overview_table_columns = apply_filters( 'picu_selection_overview_table_columns', $picu_overview_table_columns, $post->ID );
			?>
			<div class="picu-table-wrap">
				<table class="picu-selection-overview-table js-picu-selection-overview-table">
					<thead>
						<tr>
						<?php
							foreach( $picu_overview_table_columns as $column_class => $column_headline ) {
								echo '<th class="' . $column_class . '">' . $column_headline .'</th>';
							}
						?>
						</tr>
					</thead>
					<tbody>
					<?php
					$i = 1;
					foreach ( $gallery_image_ids as $gallery_image_id ) {
						$file = wp_get_attachment_image_src( $gallery_image_id, 'full' );
						$filename = pathinfo( $file[0], PATHINFO_BASENAME );
						$image = wp_get_attachment_image_src( $gallery_image_id, 'picu-small' );

						$image_classes = array();

						$image_classes[] = 'picu-selection-table-image';

						// Add classes for selected & not-selected
						if ( ! empty( $picu_collection_hashes ) AND count( $picu_multi_selected_by_no_one ) > 0 ) {
							// Check if the current image is in the "no one selected these" array
							if ( in_array( $gallery_image_id, $picu_multi_selected_by_no_one ) ) {
								$image_classes[] = 'not-selected';
							}
						}
						// If this is no multi-client collection, we do it the easy way: either selected or not
						elseif ( isset( $selection_image_ids ) AND is_array( $selection_image_ids ) ) {
							$image_classes[] = ( in_array( $gallery_image_id, $selection_image_ids ) ) ? 'selected' : 'not-selected';
						}

						// Add class for selected by all, if more than one client has actally made a selection
						if ( $picu_multi_selected_by_all_count > 1 ) {
							if ( in_array( $gallery_image_id, $picu_multi_selected_by_all ) ) {
								$image_classes[] = 'selected-by-all';
							}
						}

						// Add classes for images selected by invidual clients
						if ( ! empty( $picu_collection_hashes ) ) {
							foreach( $picu_collection_hashes as $key => $hash ) {
								$selection = get_post_meta( $post->ID, '_picu_collection_selection_' . $key, false );
								foreach( $picu_multi_selections as $key => $selection ) {
									if ( is_array( $selection) AND in_array( $gallery_image_id, $selection ) ) {
										$image_classes[] = $key;
										$image_classes[] = 'selected';
									}
								}
							}
						}

						/**
						 * Add custom classes to each row using the 'filter_options' array from above
						 * Callback function 'picu_image_{option name}' to add {option_name} as class
						 */
						foreach( $filter_options as $option_name => $option_title ) {

							// No callback for the default options
							if ( $option_name == 'all' OR $option_name == 'selected' OR $option_name == 'not-selected' ) {
								continue;
							}
							else {
								// Run the callback function 'picu_image_{option_name}'
								$function = 'picu_image_' . $option_name;
								if ( function_exists( $function ) ) {
									if ( $function( $gallery_image_id, $post ) ) {
										$image_classes[] = $option_name;
									}
								}
							}
						}

						// Remove duplicates
						$image_classes = array_unique( $image_classes );

						// Get the classes as a space separated string
						$image_classes = implode( ' ', $image_classes );

						ob_start(); ?>

						<tr class="<?php echo $image_classes; ?>" data-filename="<?php echo pathinfo( $file[0],  PATHINFO_FILENAME ); ?>" <?php if ( is_array( $selection_image_ids ) AND count( $selection_image_ids ) > 0 AND ! in_array( $gallery_image_id, $selection_image_ids ) ) { echo ' style="display: none;"'; } ?>>
							<?php
								// Iterate through table columns
								$column_count = 0;
								foreach( $picu_overview_table_columns as $key => $value ) {
									// Thumbnail column
									if ( $key == 'thumbnail' ) { ?>
										<td class="thumb"><a href="<?php echo get_the_permalink( $post->ID ) . '#' . $i; ?>" target="_blank"><img src="<?php echo $image[0]; ?>" alt="<?php echo $filename; ?>" /></a></td><?php
									}
									// Filename column
									elseif ( $key == 'file' ) {
										?><td class="file"><?php echo $filename; ?></td><?php
									}
									// Selected column
									elseif ( $key == 'approved' ) { ?>
										<td class="approved">
											<span class="approved-marker"></span>
										<?php
										if ( isset( $image_votes['image_' . $gallery_image_id] ) ) {
											$votes = $image_votes['image_' . $gallery_image_id];
											if ( $votes > 0 ) {
												echo '<span class="selection-count">' . $votes . '</span>';
											}
										}
										?>
										</td>
									<?php }
									// Third party columns
									else {
										/*
										 * Get column content via a filter
										 *
										 * Column is empty by default
										 */
										$column_content = apply_filters( 'picu_selection_overview_table_column_' . $key . '_content', '', $post->ID, $gallery_image_id, $i );
									?>
										<td class="<?php echo $key; ?>"><?php echo $column_content; ?></td>
									<?php
									}
									$column_count++;
								}
							?>
						</tr>
						<?php
							echo PHP_EOL;
							echo ob_get_clean();
							$i++;
					}
					?>
					</tbody>
				</table>
			</div>
		</div><!-- .picu-postbox-inner -->
	</div><!-- .postbox.picu-postbox -->
<?php
	echo ob_get_clean();
}


/**
 * Display approval form fields
 * 
 * @since 1.6.5
 * 
 * @param array $approval_fields The fields, containing labels and values
 * @param string $date The date, when the recipient approved the collection
 */
function picu_display_approval_fields( $approval_fields, $date ) {

	if ( ! empty( $approval_fields ) ) {
		foreach( $approval_fields as $key => $value ) {
		?>
			<div class="recipient__comment" id="approval_field_<?php echo $key; ?>"><strong><?php echo $value['label'] . ':</strong> ';
			if ( ! empty( $value['title'] ) ) {
				echo $value['title'];
			}
			else {
				echo $value['value'];
			}
		?></div>
		<?php
		}
		?>
			<div class="recipient__comment-date"><?php echo $date; ?></div>
		<?php
	}
}


/**
 * picu media upload save functionality
 *
 * @since  0.4.0
 */
function picu_save_gallery_ids( $post_id ) {

	// Check if nonce is set
	if ( ! isset( $_POST['picu_gallery_ids_nonce'] ) )
		return $post_id;

	// Verify that the nonce is valid
	if ( ! wp_verify_nonce( $_POST['picu_gallery_ids_nonce'], 'picu_gallery_ids' ) )
		return $post_id;

	// If this is an autosave, our form has not been submitted, so we don't want to do anything
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
		return $post_id;
	}

	// Check user permissions
	if ( ! current_user_can( 'edit_post', $post_id ) ) {
		return $post_id;
	}

	// Run import filter.
	// Should return true, if an import from another source occured.
	$import_done = apply_filters( 'picu_save_gallery_ids_filter', $post_id, $_REQUEST );

	// Only update gallery id meta, if import didn't happen
	if ( true !== $import_done AND ! empty( $_POST['picu_gallery_ids'] ) ) {
		// Sanitize data and put the image ids into a variable
		$picu_gallery_ids = sanitize_text_field( $_POST['picu_gallery_ids'] );

		// Save the image ID's as custom post meta
		update_post_meta( $post_id, '_picu_collection_gallery_ids', $picu_gallery_ids );
	}

	// Check if a valid share method is chosen
	if ( ! isset( $_POST['picu_collection_share_method'] ) OR ( 'picu-send-email' != $_POST['picu_collection_share_method'] AND 'picu-copy-link' != $_POST['picu_collection_share_method'] ) ) {
		return $post_id;
	}
	elseif ( 'picu-send-email' == $_POST['picu_collection_share_method'] ) {

		// Save share method
		update_post_meta( $post_id, '_picu_collection_share_method', 'picu-send-email' );

		// Check if multiple clients
		$email_addresses = $_POST['picu_collection_email_address'];
		$email_addresses = explode( ', ', $email_addresses );

		if ( is_array( $email_addresses ) AND count( $email_addresses ) > 1 ) {
	
			$email_addresses = array_filter( $email_addresses, 'picu_validate_email_address' );

			// If this collection gets reopened and send again, check if we have exsting hashes and re-use them
			$existing_hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );

			// Create an individual hash for each email-address
			$collection_hashes = [];

			foreach( $email_addresses as $address ) {
				
				if ( is_array( $existing_hashes ) AND array_search( $address, array_column( $existing_hashes, 'email' ) ) !== false ) {
					// This email address already has a hash
					$index = array_search( $address, array_column( $existing_hashes, 'email' ) );
					$hash = array_keys( $existing_hashes )[$index];
					$collection_hashes[$hash] = [
						'name' => '',
						'email' => $address,
						'status' => 'sent',
						'time' => time(),
					];
				}
				else {
					$collection_hashes[substr( md5( rand() ), 0, 10 )] = [
						'name' => '',
						'email' => $address,
						'status' => 'sent',
						'time' => time(),
					];
				}
			}

			update_post_meta( $post_id, '_picu_collection_hashes', $collection_hashes );

			// Make a string to save them
			$email_addresses = implode( ', ', $email_addresses );

			update_post_meta( $post_id, '_picu_collection_email_address', $email_addresses );

		}
		// Make sure we have one valid email address
		elseif ( is_email ( $_POST['picu_collection_email_address'] ) ) {

			$picu_collection_email_address = sanitize_email( $_POST['picu_collection_email_address'] );
			// Update the email address in the database
			update_post_meta( $post_id, '_picu_collection_email_address', $picu_collection_email_address );

			// Remove collection hashes, as this is now a single collection
			delete_post_meta( $post_id, '_picu_collection_hashes' );
			// TODO: Delete existing selections also?
		}
		else {
			delete_post_meta( $post_id, '_picu_collection_email_address' );
		}

		// Clean up the collection description
		$picu_collection_description = implode( "\n", array_map( 'sanitize_text_field', explode( "\n", $_POST['picu_collection_description'] ) ) );

		if ( ! empty( $picu_collection_description ) ) {
			// Update the collection description in the database
			update_post_meta( $post_id, '_picu_collection_description', $picu_collection_description );
		}
	}
	else {
		update_post_meta( $post_id, '_picu_collection_share_method', 'picu-copy-link' );
	}
}

add_action( 'save_post_picu_collection', 'picu_save_gallery_ids' );


/**
 * Custom Function to re-open an already sent collection
 *
 * @since 0.3.0
 */
function picu_collection_reopen() {

	// Check if a "reopen" parameter (the nonce) was set with this request
	if ( isset( $_REQUEST['reopen'] ) AND isset( $_REQUEST['post'] ) ) {

		$post_id = sanitize_key( $_REQUEST['post'] );

		// If it is, save it in a variable
		$reopen_nonce = $_REQUEST['reopen'];

		// Verify the nonce to see if it is a legitimate request
		if ( ! wp_verify_nonce( $reopen_nonce, 'picu_collection_reopen_' . $post_id ) ) {
			wp_die ( __( 'Security check failed!', 'picu' ) );
		} else {
			// Check if individual client should be reopened
			// Only allow it, if the collection status is sent or approved
			if ( ! empty( $_GET['client'] ) AND ( get_post_status( $post_id ) == 'approved' OR get_post_status( $post_id ) == 'sent' ) ) {

				$client = sanitize_key( $_GET['client'] );
				$hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );
				
				if ( array_key_exists( $client, $hashes ) AND $hashes[$client]['status'] == 'approved' ) {
					$hashes[$client]['status'] = 'sent';
					$hashes[$client]['time'] = time();
					update_post_meta( $post_id, '_picu_collection_hashes', $hashes );

					// Check if the general collection status needs to be updated as well
					if ( get_post_status( $post_id ) == 'approved' ) {
						picu_update_post_status( $post_id, 'sent' );
						picu_update_collection_history( $post_id, 'reopened' );
					}
				}
				else {
					wp_die( '👀 ' . __( 'Client ID not found.', 'picu' ) );
				}

				return;
			}

			if ( get_post_status( $post_id ) == 'delivered' ) {
				picu_update_post_status( $post_id, 'delivery-draft' );
				picu_update_collection_history( $post_id, 'reopened' );
			}
			elseif ( get_post_status( $post_id ) == 'approved' ) {
				picu_update_post_status( $post_id, 'sent' );

				// Update multi-client collection
				$collection_hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );
				if ( ! empty( $collection_hashes ) ) {
					$updated_collection_hashes = [];
					foreach( $collection_hashes as $key => $hash ) {
						$hash['status'] = 'sent';
						$hash['time'] = time();
						$updated_collection_hashes[$key] = $hash; 
					}
					update_post_meta( $post_id, '_picu_collection_hashes', $updated_collection_hashes );
				}

				picu_update_collection_history( $post_id, 'reopened' );
			}
			else {
				picu_update_post_status( $post_id, 'draft' );
				
				// Update multi-client collection
				$collection_hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );
				if ( ! empty( $collection_hashes ) ) {
					$updated_collection_hashes = [];
					foreach( $collection_hashes as $key => $hash ) {
						$hash['status'] = 'draft';
						$hash['time'] = time();
						$updated_collection_hashes[$key] = $hash; 
					}
					update_post_meta( $post_id, '_picu_collection_hashes', $updated_collection_hashes );
				}

				picu_update_collection_history( $post_id, 'reopened' );
			}
		}
	}
}

add_action( 'wp_loaded', 'picu_collection_reopen' );


/**
 * Remove the default "post-submit" metabox
 *
 * @since 0.3.0
 */
function picu_remove_submit_metabox() {

	remove_meta_box( 'submitdiv', 'picu_collection', 'side' );

}
add_action( 'admin_menu', 'picu_remove_submit_metabox' );


/**
 * Replace regular "Edit Collection" title with actual title
 */
function picu_replace_edit_screen_title() {
	global $post, $title, $action;

	if ( empty( $post ) OR empty( $action ) ) {
		return;
	}

	if ( 'edit' == $action AND 'picu_collection' == $post->post_type AND ( 'approved' == $post->post_status OR 'sent' == $post->post_status ) ) {
		if ( empty( $post->post_title ) ) {
			$title = __( '(no title)', 'picu' );
		} else {
			$title = $post->post_title;
		}
	}
}

add_action( 'admin_head', 'picu_replace_edit_screen_title' );


/**
 * Duplicate collection
 *
 * @since 0.9.4
 *
 * @param int $post_id The collection ID
 */
function picu_duplicate_collection( $post_id, $selected = false ) {

	// Get image ids and description from post meta
	$custom_meta = get_post_custom( $post_id );

	// Check if it is actually a collection, and if the required meta is available
	if ( 'picu_collection' == get_post_type( $post_id ) AND isset( $custom_meta['_picu_collection_gallery_ids'] ) ) {

		$gallery_ids = $custom_meta['_picu_collection_gallery_ids'];
		if ( isset( $custom_meta['_picu_collection_description'] ) ) {
			$collection_description = $custom_meta['_picu_collection_description'];
		}

		$arg = array(
			'post_type' => 'picu_collection',
		);

		if ( ! empty( get_the_title( $post_id ) ) ) {
			$arg['post_title'] = __( 'Copy of', 'picu' ) . ' ' . get_the_title( $post_id );
		}

		// Create new collection
		$new_id = wp_insert_post( $arg );
	}

	// If we succesfully created the new collection...
	if ( isset( $new_id ) AND ! empty( $new_id ) ) {

		// Add the description to the new collection
		if ( isset( $collection_description ) AND is_array( $collection_description ) ) {
			add_post_meta( $new_id, '_picu_collection_description', $collection_description[0] );
		}

		// Prepare Copying / duplicating images
		$upload_dir = wp_upload_dir();
		$collection_image_dir = trailingslashit( $upload_dir['basedir'] ) . 'picu/collections/' . $post_id;
		$new_collection_image_dir = trailingslashit( $upload_dir['basedir'] ) . 'picu/collections/' . $new_id;

		if ( ! is_dir( $new_collection_image_dir ) ) {
			mkdir( $new_collection_image_dir, 0755 );
		}

		// For multi client collections
		$picu_collection_hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );

		if ( ! empty( $picu_collection_hashes ) ) {
			$image_ids = [];
			foreach( $picu_collection_hashes as $key => $hash ) {
				$selection = get_post_meta( $post_id, '_picu_collection_selection_' . $key, true );
				if ( ! empty( $selection['selection'] ) ) {
					$image_ids = array_merge( $image_ids, $selection['selection'] );
				}
			}
			$selection = array_unique( $image_ids );
		}
		// For single collections
		else {
			$selection = get_post_meta( $post_id, '_picu_collection_selection', true );
			$selection = $selection['selection'];
		}

		// Get final image ids to be copied
		if ( 'selected' == $selected ) {
			$image_ids = $selection;
		}
		elseif ( 'not-selected' == $selected ) {
			$all_image_ids = explode( ',', $gallery_ids[0] );
			$image_ids = array_diff( $all_image_ids, $selection );
		}
		else {
			$image_ids = explode( ',', $gallery_ids[0] );
		}

		// Load images
		$images = get_posts( array(
			'include' => $image_ids,
			'post_status' => 'any',
			'post_type' => 'attachment',
			'post_mime_type' => 'image',
			'order' => 'ASC'
		) );

		$new_gallery_ids = array();

		foreach( $images as $image ) {

			// Get original file path
			$original_image_path = get_attached_file( $image->ID );

			// Get file name
			$image_name = basename( $original_image_path );

			// Get meta data from original file
			$original_meta = wp_get_attachment_metadata( $image->ID );

			// Copy original file
			if ( copy( $original_image_path, $new_collection_image_dir . '/' . $image_name ) ) {

				// Copy other available sizes as well
				foreach( $original_meta['sizes'] as $thumbnail ) {
					// Check if there is acutally a file
					if ( isset( $thumbnail['file'] ) AND ! empty( $thumbnail['file'] ) ) {
						copy( $collection_image_dir . '/' . $thumbnail['file'], $new_collection_image_dir . '/' . $thumbnail['file'] );
					}
				}

				// Prepare file attachment
				$filetype = wp_check_filetype( basename( $image_name ), null );

				$attachment = array(
					'guid'           => $new_collection_image_dir . '/' . $image_name,
					'post_mime_type' => $filetype['type'],
					'post_title'     => preg_replace( '/\.[^.]+$/', '', basename( $image_name ) ),
					'post_content'   => '',
					'post_status'    => 'inherit'
				);

				// Prevent new thumbnails from being created during this process
				add_filter( 'intermediate_image_sizes_advanced', '__return_false' );

				// Prevent creation of scaled image
				add_filter( 'big_image_size_threshold', '__return_false' );

				// Insert file as new attachment
				$attach_id = wp_insert_attachment( $attachment, $new_collection_image_dir . '/' . $image_name, $new_id );

				// wp_generate_attachment_metadata() depends on this file
				require_once( ABSPATH . 'wp-admin/includes/image.php' );

				// Generate the metadata for the new attachment
				$attach_data = wp_generate_attachment_metadata( $attach_id, $new_collection_image_dir . '/' . $image_name );

				// Use the sizes from the original file
				$attach_data['sizes'] = $original_meta['sizes'];

				// Update attachment meta data
				wp_update_attachment_metadata( $attach_id, $attach_data );

				// Add new attachment id to our array
				$new_gallery_ids[] = $attach_id;

			}
			else {
				// Cleanup: delete the new collection we just created
				wp_delete_post( $new_id, true );

				// Report error to the user
				$error = error_get_last();
				picu_add_notification( 'picu_duplication_failed', 'notice notice-error is-dismissible', __( 'Duplication failed with the following error: ', 'picu' ) . $error['message'] );

				// Redirect to collection overview
				wp_redirect( admin_url( 'edit.php?post_type=picu_collection&picu_notification=1' ) );
				exit;
			}
		}

		// Create string with image IDs
		$new_gallery_ids = implode( ',', $new_gallery_ids );

		// Save image IDs for our new collection
		add_post_meta( $new_id, '_picu_collection_gallery_ids', $new_gallery_ids );

		// Redirect to newly created collection edit screen
		wp_redirect( admin_url( 'post.php?action=edit&post=' . $new_id ) );
		exit;
	}
	else {
		// Redirect to picu collection overview, set parameter to show error
		wp_redirect( admin_url( 'edit.php?post_type=picu_collection&picu=duplication-error' ) );
		exit;
	}
}

// Run the collection function, if the correct parameter is set
if ( isset( $_REQUEST['picu_duplicate_collection'] ) AND ! empty( $_REQUEST['picu_duplicate_collection'] ) AND wp_verify_nonce( $_REQUEST['picu_duplication_nonce'], 'picu_duplicate_collection') ) {
	if ( isset( $_REQUEST['picu_duplicate_collection_selected'] ) AND '1' == $_REQUEST['picu_duplicate_collection_selected'] ) {
		picu_duplicate_collection( absint( $_REQUEST['picu_duplicate_collection'] ), 'selected' );
	}
	elseif ( isset( $_REQUEST['picu_duplicate_collection_not_selected'] ) AND '1' == $_REQUEST['picu_duplicate_collection_not_selected'] ) {
		picu_duplicate_collection( absint( $_REQUEST['picu_duplicate_collection'] ), 'not-selected' );
	}
	else {
		picu_duplicate_collection( absint( $_REQUEST['picu_duplicate_collection'] ) );
	}
}

// Display error message, if duplication failed
if ( isset( $_REQUEST['picu'] ) AND 'duplication-error' == $_REQUEST['picu'] ) {
	function picu_duplication_error_notice() { ?>
		<div class="error notice is-dismissible">
			<p><?php _e( 'Duplication failed. Please try again.', 'picu' ); ?></p>
		</div>
	<?php }

	add_action( 'admin_notices', 'picu_duplication_error_notice' );
}


/**
 * Add duplicate as row action item
 *
 * @since 0.9.4
 */
function picu_add_duplicate_link( $actions, $post ) {

	// TODO: Add multi client support

	if ( 'picu_collection' == $post->post_type AND ! empty( get_post_meta( $post->ID, '_picu_collection_gallery_ids', true ) ) AND $post->post_status != 'delivery-draft' AND $post->post_status != 'delivered' ) {

		if ( $post->post_status == 'approved' AND picu_get_selection_count( $post->ID ) > 0 ) {
			ob_start();
		?>
			<a class="js-picu-duplicate" data-id="<?php echo $post->ID; ?>" href="<?php echo wp_nonce_url( admin_url( 'post.php?picu_duplicate_collection=' . $post->ID ), 'picu_duplicate_collection', 'picu_duplication_nonce' ); ?>"><?php _e( 'Duplicate', 'picu' ); ?>&hellip;</a>
		<?php
			echo picu_get_duplication_modal( $post->ID );
			$actions['picu_duplication'] = ob_get_clean();
		}
		else {
			$actions['picu_duplication'] = '<a href="' . wp_nonce_url( admin_url( 'post.php?picu_duplicate_collection=' . $post->ID ), 'picu_duplicate_collection', 'picu_duplication_nonce' ) . '">'. __( 'Duplicate', 'picu' ) . '</a>';
		}
	}

	return $actions;
}

add_filter( 'post_row_actions', 'picu_add_duplicate_link', 10, 2 );



/**
 * Custom function to change post status to delivery-draft
 *
 * @since 1.5.0
 */
function picu_collection_delivery() {

	// Check if a "reopen" parameter (the nonce) was set with this request
	if ( isset( $_REQUEST['delivery'] ) AND isset( $_REQUEST['delivery'] ) ) {

		// If it is, save it in a variable
		$delivery_nonce = $_REQUEST['delivery'];

		// Verify the nonce to see if it is a legitimate request
		if ( ! wp_verify_nonce( $_REQUEST['delivery'] ) ) {
			wp_die( __( 'Security check failed!', 'picu' ) );

		} else {
			if ( ! empty( $_REQUEST['post'] ) ) {
				$post_id = sanitize_key( $_REQUEST['post'] );
				picu_update_post_status( $post_id, 'delivery-draft' );
				picu_update_collection_history( $post_id, 'preparing_delivery' );
			}
			else {
				// Create new delivery draft
				$post_id = wp_insert_post( array(
					'post_type' => 'picu_collection',
					'post_status' => 'delivery-draft'
				) );
			}

			wp_redirect( admin_url( 'post.php?action=edit&post=' . $post_id ) );
			exit;
		}
	}
}

add_action( 'wp_loaded', 'picu_collection_delivery' );


/**
 * Disable the block editor (aka Gutenberg) for picu collections
 *
 * @since 1.2.2
 */
function picu_disable_block_editor( $use_block_editor, $post_type ) {

	if ( $post_type === 'picu_collection' ) {
		return false;
	}

	return $use_block_editor;

}

add_filter( 'gutenberg_can_edit_post_type', 'picu_disable_block_editor', 10, 2 );
add_filter( 'use_block_editor_for_post_type', 'picu_disable_block_editor', 10, 2 );


/**
 * Display duplication modal
 *
 * @since 1.3.4
 *
 * @param $post_id - The collection id
 */
function picu_get_duplication_modal( $post_id ) {

	// TODO: Make work for mutli client collections

	// Get number of images
	$picu_collection_gallery_ids = get_post_meta( $post_id, '_picu_collection_gallery_ids', true );
	$total_images_num = count( explode( ',', $picu_collection_gallery_ids ) );

	// Get number of selected images
	$selected_images_num = picu_get_selection_count( $post_id );

	// Get numner of not-selected images
	$not_selected_images_num = $total_images_num - $selected_images_num;

	ob_start();
	?>

	<div class="picu-modal is-hidden" id="js-picu-duplicate-<?php echo $post_id; ?>">
		<div class="picu-modal-inner">
			<div class="picu-modal-content">
				<h3><?php _e( 'Duplicate', 'picu' ); ?>&hellip;</h3>
				<p><input type="radio" class="js-duplication-radio ays-ignore" id="picu_duplicate_all_<?php echo $post_id; ?>" data-id="<?php echo $post_id; ?>" name="picu_duplicate_<?php echo $post_id; ?>" value="all" checked="checked" /> <label for="picu_duplicate_all_<?php echo $post_id; ?>"><?php _e( 'All images', 'picu' ); ?> <em>(<?php echo $total_images_num; ?>)</em></label></p>
				<p><input type="radio" class="js-duplication-radio ays-ignore" id="picu_duplicate_selected_<?php echo $post_id; ?>" data-id="<?php echo $post_id; ?>" name="picu_duplicate_<?php echo $post_id; ?>" value="selected" /> <label for="picu_duplicate_selected_<?php echo $post_id; ?>"><?php _e( 'Selected images only', 'picu' ); ?> <em>(<?php echo $selected_images_num; ?>)</em></label></p>
				<?php if ( $not_selected_images_num > 0 ) { ?>
				<p><input type="radio" class="js-duplication-radio ays-ignore" id="picu_duplicate_not_selected_<?php echo $post_id; ?>" data-id="<?php echo $post_id; ?>" name="picu_duplicate_<?php echo $post_id; ?>" value="not-selected" /> <label for="picu_duplicate_not_selected_<?php echo $post_id; ?>"><?php _e( 'Unselected images only', 'picu' ); ?> <em>(<?php echo $not_selected_images_num; ?>)</em></label></p>
				<?php } ?>
				<input type="hidden" class="ays-ignore" name="picu_duplication_url_<?php echo $post_id; ?>" value="<?php echo wp_nonce_url( admin_url( 'post.php?picu_duplicate_collection=' . $post_id ), 'picu_duplicate_collection', 'picu_duplication_nonce' ); ?>" />
				<div class="picu-modal-actions">
					<a class="button button-primary js-picu-duplication-button-<?php echo $post_id; ?>" href="<?php echo wp_nonce_url( admin_url( 'post.php?picu_duplicate_collection=' . $post_id ), 'picu_duplicate_collection', 'picu_duplication_nonce' ); ?>"><?php _e( 'Duplicate', 'picu' ); ?></a>
					<a class="button picu-cancel-modal js-picu-cancel-modal" href="#"><?php _e( 'Cancel', 'picu' ); ?></a>
				</div>
			</div>
		</div>
	</div>

	<?php
	return ob_get_clean();
}