<?php
/**
 * Template for picu HTML Emails
 *
 * @since 0.7.3
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


function picu_get_email_template( $mail_content ) {

	global $post;

	if ( ! isset( $mail_content['footer_notice'] ) ) {
		$mail_content['footer_notice'] = '';
	}

	if ( ! isset( $mail_content['button_text'] ) ) {
		$mail_content['button_text'] = __( 'Click here', 'Default button text in picu emails', 'picu' );
	}

	if ( ! isset( $mail_content['title'] ) ) {
		$mail_content['title'] = __( 'Untitled', 'picu' );
	}

	// Filter to change the logo
	$mail_content['logo'] = '';
	$logo_src = apply_filters( 'picu_logo', $mail_content['logo'] );
	if ( ! empty( $logo_src ) ) {
		$mail_content['logo'] = '<p style="margin-bottom: 20px; text-align: center;"><a href="' . esc_url( home_url( '/' ) ) . '"><img style="height: 45px; width: auto;" class="logo" src="' . $logo_src . '" /></a></p>';
	}

	if ( ! isset( $mail_content['user_email'] ) ) {
		$mail_content['user_email'] = '';
	}

	if ( ! isset( $mail_content['message_text'] ) ) {
		$mail_content['message_text'] = '';
	}
	$mail_content['message_text'] = apply_filters( 'picu_email_message_text', $mail_content['message_text'] );

	if ( ! isset( $mail_content['mail_html'] ) ) {
		$mail_content['message_html'] = str_replace( "\n", '<br>', $mail_content['message_text'] );

		$mail_content['message_html'] = apply_filters( 'picu_email_message_html', $mail_content['message_html'] );
	}

	// Check if collection is password protected
	$mail_content['password'] = '';

	// Allow password in emails? Defaults to true.
	$options = get_option( 'picu_settings' );
	$send_password_in_email = true;
	
	if ( isset( $options['send_password'] ) AND $options['send_password'] == 'off' ) {
		$send_password_in_email = false;
	}

	// Posibility to override with filter
	$send_password_in_email = apply_filters( 'picu_send_password_in_email', $send_password_in_email );

	if ( isset( $_POST['post_password'] ) AND ! empty( $_POST['post_password'] ) AND $send_password_in_email ) {
		$mail_content['password'] = __( 'Password:', 'picu' ) . ' ' . stripslashes( sanitize_text_field( $_POST['post_password'] ) );
	}

	// Load picu settings
	$options = get_option( 'picu_settings' );

	// Do not send HTML emails
	if ( isset( $options['send_html_mails'] ) AND 'on' != $options['send_html_mails'] ) {
		$mail_template = $mail_content['message_text'];

		if ( ! empty( $mail_content['collection_hashes'] ) AND $post->post_status != 'delivery-draft' ) {
			$mail_template = [];
			foreach( $mail_content['collection_hashes'] as $hash => $data ) {
				$mail_template[$data['email']] = $mail_content['message_text'] . "\n\n" . esc_url( add_query_arg( 'ident', $hash, $mail_content['url'] ) );

				if ( ! empty( $mail_content['password'] ) ) {
					$mail_template[$data['email']] .= "\n\n" . $mail_content['password'];
				}
			}
		}
		else {
			$mail_template = $mail_content['message_text'] . "\n\n" . $mail_content['url'];

			if ( ! empty( $mail_content['password'] ) ) {
				$mail_template .= "\n\n" . $mail_content['password'];
			}
		}
	// Send HTML email
	} else {
		// Create individual templates for every email address
		// Do not take hashes into account on delivery collecitons!
		if ( ! empty( $mail_content['collection_hashes'] ) AND $post->post_status != 'delivery-draft' ) {
			$mail_template = [];
			foreach( $mail_content['collection_hashes'] as $hash => $data ) {
				$mail_template[$data['email']] = picu_build_html_email_template( $mail_content, $options, $hash );
			}
		}
		else {
			$mail_template = picu_build_html_email_template( $mail_content, $options );
		}
	}

	// Return the content
	return $mail_template;
}




function picu_build_html_email_template( $mail_content, $options, $hash = '' ) {
	
	// Load the template file according to theme setting
	if ( isset( $options['theme'] ) AND $options['theme'] == 'light' ) {
		$mail_template = file_get_contents( PICU_PATH . '/backend/includes/email-templates/email-template-light-inline.html' );

		$picu_logo_uri = PICU_URL . '/backend/images/picu_logo_dark.png';
		$primary_base_color = '#2f92a7';
		$primary_color = '#2f92a7';
	}
	else {
		$mail_template = file_get_contents( PICU_PATH . '/backend/includes/email-templates/email-template-dark-inline.html' );

		$picu_logo_uri = PICU_URL . '/backend/images/picu_logo_light.png';
		$primary_base_color = '#7ad03a';
		$primary_color = '#7ad03a';
	}

	// Filter to change the primary color
	$primary_color = apply_filters( 'picu_primary_color', $primary_color );

	if ( isset( $options['picu_love'] ) AND 'on' == $options['picu_love'] ) {
		$picu_love = '<p>powered by</p><a href="https://picu.io/"><img src="' . $picu_logo_uri . '" width="80" style="width: 80px; height: auto;" alt="picu Logo"></a>';
	} else {
		$picu_love = '';
	}

	if ( ! empty( $hash ) ) {
		$mail_content['url'] = esc_url( add_query_arg( 'ident', $hash, $mail_content['url'] ) );
	}

	// Replace placeholders in template with actual strings
	$mail_template = str_replace(
		array(
			'{footer_notice}',
			'{button_text}',
			'{title}',
			'{logo}',
			'{user_email}',
			'{message_text}',
			'{message_html}',
			'{url}',
			'{password}',
			'{picu_love}',
			$primary_base_color
		),
		array(
			$mail_content['footer_notice'],
			$mail_content['button_text'],
			$mail_content['title'],
			$mail_content['logo'],
			$mail_content['user_email'],
			$mail_content['message_text'],
			$mail_content['message_html'],
			$mail_content['url'],
			$mail_content['password'],
			$picu_love,
			$primary_color
		),
		$mail_template
	);

	return $mail_template;
}
