<?php
/**
 * Fix stuff from third parties
 *
 * @since 1.1.0
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


/**
 * Remove picu collections from Yoast xml sitemaps
 *
 * @since 0.9.4
 */
function picu_remove_from_wpseo_sitemap( $exclude = false, $post_type ) {

	if ( 'picu_collection' == $post_type ) {
		return true;

	}
	return false;
}

add_filter( 'wpseo_sitemap_exclude_post_type', 'picu_remove_from_wpseo_sitemap', 10, 2 );


/**
 * Remove picu images from attachment sitemap
 *
 * @since 1.1.0
 */
function picu_remove_attachments_from_yoast_sitemap( $output, $url ) {

	if ( isset( $url['images'][0]['src'] ) AND strpos( $url['images'][0]['src'], '/picu/' ) ) {
		return '';
	}
	else {
		return $output;
	}

}

add_filter( 'wpseo_sitemap_url', 'picu_remove_attachments_from_yoast_sitemap', 10, 2 );


/**
 * Remove picu collections from Yoast accessible post types list (whatever that means)
 */
function picu_remove_from_yoast( $post_types ) {

	unset( $post_types['picu_collection'] );

	return $post_types;
}

add_filter( 'wpseo_accessible_post_types', 'picu_remove_from_yoast' );


/**
 * Add picu collections to the list of Yoast excluded post types
 */
function  picu_exclude_from_yoast( $excluded ) {

	$excluded[] = 'picu_collection';
	return $excluded;

}

// add_filter( 'wpseo_indexable_excluded_post_types', 'picu_exclude_from_yoast' );


// Enable for testing purposes only!
// add_filter('wpseo_enable_xml_sitemap_transient_caching', '__return_false');


/**
 * Remove picu collections from sitemaps created with the SEO Framework plugin
 *
 * @since 1.3.0
 */
function picu_remove_collections_from_seo_framework_sitemap() {
	return array( 'picu_collection' );
}

add_filter( 'the_seo_framework_sitemap_exclude_cpt', 'picu_remove_collections_from_seo_framework_sitemap' );


/**
 * Remove the SEO Framework meta box from picu collections
 *
 * @since 1.3.0
 */
function picu_remove_seo_framework_meta_box() {
	$current_screen = get_current_screen();

	if ( $current_screen->id == 'picu_collection' ) {
		add_filter( 'the_seo_framework_seobox_output', '__return_false' );
	}
}

add_action( 'current_screen', 'picu_remove_seo_framework_meta_box' );


/**
 * Disable code injection by NextGen Gallery
 *
 * @since 1.0.4
 */
function picu_disable_ngg_resource_manager() {
	if ( isset( $_SERVER['REQUEST_URI'] ) AND preg_match( "/post_type=picu_collection/", $_SERVER['REQUEST_URI'] ) ){
		return false;
	}
}

add_filter( 'run_ngg_resource_manager', 'picu_disable_ngg_resource_manager' );


/**
 * Prevent Autoptimize from "optimizing" aka breaking stuff
 *
 * @since 1.3.2
 */
function picu_no_autoptimize() {
	global $post;

	if ( $post->post_type == 'picu_collection' ) {
		return true;
	}

	return false;
}

add_filter( 'autoptimize_filter_noptimize', 'picu_no_autoptimize' );


/**
 * Deactivate Real Media Library for picu collections
 *
 * @since 1.3.5
 */
function picu_deactivate_real_media_library( $active ) {

	global $post;

	if ( isset( $post->post_type ) AND $post->post_type == 'picu_collection' ) {
		$active = false;
	}

	return $active;

}

add_filter( 'RML/Active', 'picu_deactivate_real_media_library' );


/**
 * Do not use Jetpacks photon for picu images
 *
 * @since 1.4.5
 */
function picu_no_photon() {
	if ( get_post_type() == 'picu_collection' ) {
		add_filter( 'jetpack_photon_skip_image', '__return_true' );
	}
}
 
add_action( 'wp', 'picu_no_photon' );


/**
 * Remove picu images from Media Library Assistant shortcode generated galleries
 * 
 * @since 1.4.13
 */
function picu_remove_images_from_media_library_galleries( $filtered, $attachments ) {

	$filtered = array();

	$filtered = array_filter( $attachments, function( $attachment ) {
		if ( isset( $attachment->post_parent ) AND get_post_type( $attachment->post_parent ) == 'picu_collection' ) {
			return false;
		}
		return true;
	});
	
	return $filtered;
}

add_filter( 'mla_gallery_the_attachments', 'picu_remove_images_from_media_library_galleries', 10, 2 );