<?php
/**
 * Picu front end template
 *
 * @since 0.3.0
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


/**
 * Returns template file
 *
 * @since 0.3.0
 */

function picu_load_template( $single_template ) {

	global $post;

	if ( isset( $post ) AND $post->post_type == 'picu_collection' ) {
		if ( file_exists( PICU_PATH . 'frontend/picu-app.php' ) ) {
			$single_template = PICU_PATH . 'frontend/picu-app.php';
		}
	}

	return $single_template;
}

add_filter( 'template_include', 'picu_load_template', 99 );


/**
 * Gather picu body classes
 *
 * @since 0.7.3
 */
function get_picu_body_classes() {

	$picu_body_classes = array();

	// Add body class for lazyloading
	// Legacy M&C fix
	if ( defined( 'PICU_MARK_COMMENT_VERSION' ) AND version_compare( PICU_MARK_COMMENT_VERSION, '1.0.2' ) < 0 ) {
 
	} else {
		$picu_body_classes[] = 'lazyloading';
	}

	global $post;

	if ( ! empty( $post->ID ) ) {
		$picu_body_classes[] = 'collection-id-' . $post->ID;
	}

	$ident = '';
	if ( ! empty( $_GET['ident'] ) ) {
		$ident = sanitize_key( $_GET['ident'] );
	}

	$picu_body_classes[] = 'status-' . picu_get_collection_status( $post->ID, $ident );

	/**
	 * Add class, when user is logged in
	 */
	if ( is_user_logged_in() AND current_user_can( picu_capability() ) ) {
		$picu_body_classes[] = 'user-logged-in';
	}

	/**
	 * Add class, when picu admin bar is visible
	 */
	if (  apply_filters( 'picu_show_admin_bar', true ) ) {
		$picu_body_classes[] = 'has-picu-admin-bar';
	}

	/**
	 * Add filter to modify body classes
	 */
	$picu_body_classes = apply_filters( 'picu_body_classes', $picu_body_classes );

	return $picu_body_classes;

}


/**
 * Echo picu body classes
 *
 * @since 0.7.3
 */
function picu_body_classes() {

	$picu_body_classes = get_picu_body_classes();

	if ( count( $picu_body_classes ) > 0 ) {
		echo sprintf( ' class="%s"', implode( ' ', $picu_body_classes ) );
	}

}


/**
 * Load backbone templates
 *
 * @since 0.7.2
 */
function picu_load_backbone_templates() {

	$templates = array(
		'collection-info'	=> PICU_PATH . 'frontend/js/templates/picu-collection-info.php',
		'status-bar'		=> PICU_PATH . 'frontend/js/templates/picu-status-bar.php',
		'gallery-item'		=> PICU_PATH . 'frontend/js/templates/picu-gallery-item.php',
		'lightbox'			=> PICU_PATH . 'frontend/js/templates/picu-lightbox.php',
		'send-selection'	=> PICU_PATH . 'frontend/js/templates/picu-send-selection.php',
		'approved'			=> PICU_PATH . 'frontend/js/templates/picu-approved.php'
	);

	$templates = apply_filters( 'picu_load_backbone_templates', $templates );

	return $templates;
}


/**
 * Load collections, models & views
 *
 * @since 0.7.2
 */
function picu_load_cmv() {

	$cmv = array(
		'picu-state' =>				PICU_URL . 'frontend/js/models/picu-state.js?v=' . PICU_VERSION,
		'collection-info-view' =>	PICU_URL . 'frontend/js/views/collection-info-view.js?v=' . PICU_VERSION,
		'status-bar-view' =>		PICU_URL . 'frontend/js/views/status-bar-view.js?v=' . PICU_VERSION,
		'single-image' =>			PICU_URL . 'frontend/js/models/single-image.js?v=' . PICU_VERSION,
		'picu-collection' =>		PICU_URL . 'frontend/js/collections/picu-collection.js?v=' . PICU_VERSION,
		'gallery-view' =>			PICU_URL . 'frontend/js/views/gallery-view.js?v=' . PICU_VERSION,
		'single-image-view' =>		PICU_URL . 'frontend/js/views/single-image-view.js?v=' . PICU_VERSION,
		'lightbox-view' =>			PICU_URL . 'frontend/js/views/lightbox-view.js?v=' . PICU_VERSION,
		'send-selection-view' =>	PICU_URL . 'frontend/js/views/send-selection-view.js?v=' . PICU_VERSION,
		'approved-view' =>			PICU_URL . 'frontend/js/views/approved-view.js?v=' . PICU_VERSION,
	);

	$cmv = apply_filters( 'picu_load_cmv', $cmv );

	return $cmv;
}


/**
 * Load front end stylesheet
 *
 * @since 0.7.0
 */
function picu_load_styles() {

	$styles = array(
		'picu' => PICU_URL . 'frontend/css/picu-dark.css',
		'picu-print' => PICU_URL . 'frontend/css/picu-print.css',
	);

	$styles = apply_filters( 'picu_load_styles', $styles );

	$styles_output = '';

	foreach( $styles as $name => $url ) {
		$styles_output .= "\n\t\t" . '<link href="' . $url . '?v=' . PICU_VERSION . '" rel="stylesheet" media="';
		if ( 'picu-print' == $name ) {
			$styles_output .= 'print';
		}
		else {
			$styles_output .= 'screen';
		}
		$styles_output .= '" />';
	}

	$styles_output = apply_filters( 'picu_styles_output', $styles_output );

	echo $styles_output . "\n";
}


/**
 * Load stylesheet depending on settings
 *
 * @since 0.7.0
 */
function picu_theme_options( $styles ) {

	$options = get_option( 'picu_settings' );

	if ( isset( $options['theme'] ) AND $options['theme'] == 'light' ) {
		$styles['picu'] = PICU_URL . 'frontend/css/picu-light.css';
	}

	return $styles;
}

add_filter( 'picu_load_styles', 'picu_theme_options', 10, 1 );



/**
 * Get JSON formated image collection used in the front end
 *
 * @since 1.5.0
 *
 * @param $image_ids – the image ids; $post – the picu collection object
 * @return string, javascript objects, containing the image collection
 *
 */

function picu_get_image_collection( $image_ids, $post = '' ) {

	if ( empty( $post ) ) {
		$post = get_post();
	}

	// Set up attachment requests
	$orderby = 'post__in';
	$order = 'ASC';

	// Load attachments
	$_attachments = get_posts( array( 'include' => $image_ids, 'post_status' => 'any', 'post_type' => 'attachment', 'post_mime_type' => 'image', 'order' => $order, 'orderby' => $orderby ) );

	$attachments = array();
	foreach ( $_attachments as $key => $val ) {
		$attachments[$val->ID] = $_attachments[$key];
	}

	// If we don't get attachments, this is as far as we go
	if ( empty( $attachments ) ) {
		return '[ ]';
	}

	// Prepare backbone image collection
	$i = 0;
	$imgnum = 1;
	$image_collection = array();
	$orientation = '';

	// Create image objects
	foreach ( $attachments as $id => $attachment ) {

		// Calculate image orientiation
		$image_meta = wp_get_attachment_metadata( $id );

		if ( isset( $image_meta['height'], $image_meta['width'] ) ) {
			$orientation = ( $image_meta['height'] > $image_meta['width'] ) ? 'portrait' : 'landscape';
		}

		// Get image description
		if ( trim( $attachment->post_excerpt ) ) {
			$image_description = htmlspecialchars( $attachment->post_excerpt, ENT_QUOTES, 'UTF-8' );
			$image_description = preg_replace( "/\r|\n|\t/", "", $image_description );
		} else {
			$image_description = '';
		}

		// Get attachment URLs
		$image_name = wp_get_attachment_image_src( $attachment->ID, 'full' );
		$image_path = wp_get_attachment_image_src( $attachment->ID, 'picu-large' );
		$image_path_small = wp_get_attachment_image_src( $attachment->ID, 'picu-small' );

		if ( function_exists( 'wp_get_attachment_image_srcset' ) ) {
			$image_path_srcset = ( wp_get_attachment_image_srcset( $attachment->ID, 'picu-large' ) ) ?: '';
			$image_path_small_srcset = ( wp_get_attachment_image_srcset( $attachment->ID, 'picu-small' ) ) ?: '';
		}

		// Set size attribute, for use with srcset
		$size = apply_filters( 'picu_gallery_iteme_size_attr', '(min-width: 499px) 340px, 468px' );

		// Set lightbox size attribute; do not upscale the image: set natural image width as max
		if ( isset( $image_meta['width'] ) ) {
			$size_lightbox = $image_meta['width'] . 'px';
		}
		else {
			$size_lightbox = '100vw';
		}

		// Set selected attribute, if a selection has been (auto) saved
		$temp = get_post_meta( $post->ID, '_picu_collection_selection', true );

		// Overwrite with user selection
		// TODO: Also account for cookie
		if ( ! empty( $_GET['ident'] ) ) {
			$hashes = get_post_meta( get_the_ID(), '_picu_collection_hashes', true );

			if ( is_array( $hashes ) AND array_key_exists( $_GET['ident'], $hashes ) ) {
				$temp = get_post_meta( $post->ID, '_picu_collection_selection_' . $_GET['ident'], true );
			}
		}

		if ( isset( $temp['selection'] ) and is_array( $temp['selection'] ) ) {
			$selection = $temp['selection'];
			$selected = ( in_array( $attachment->ID, $selection ) ) ? true : false;
		} else {
			$selected = false;
		}

		// Get markers
		if ( isset( $temp['markers']['id_'.$id] ) AND ! empty( $temp['markers']['id_'.$id] ) ) {

			$markers = array();

			foreach( $temp['markers']['id_'.$id] as $key => $value ) {
				$markers[$key] = $value;
			}

			// See function below
			array_walk_recursive( $markers, 'picu_encode_marker_comment' );
		}
		else {
			$markers = '';
		}

		// Remove parameters from iamge name (eg. when Jetpacks photon is used)
		$image_title = strtok( basename( $image_name[0] ), '?' );

		// Legacy M&C fix for new way of displaying image name
		// Using a regular string for M&C version older than 1.0.2
		if ( defined( 'PICU_MARK_COMMENT_VERSION' ) AND version_compare( PICU_MARK_COMMENT_VERSION, '1.0.2' ) < 0 ) {
			$title = $image_title;
		}
		else {
			$title = array( 'number' => $imgnum, 'filename' => $image_title );
		}

		$current_image = array(
			'number' => $imgnum,
			'imageID' => $attachment->ID,
			'title' => $title,
			'description' => $image_description,
			'imagePath' => $image_path[0],
			'imagePath_small' => $image_path_small[0],
			'imagePath_original' => $image_name[0],
			'imagePath_srcset' => $image_path_srcset,
			'imagePath_small_srcset' => $image_path_small_srcset,
			'size' => $size,
			'size_lightbox' => $size_lightbox,
			'orientation' => $orientation,
			'selected' => $selected,
			'markers' => $markers
		);

		/*
		 * Add filter to modify each individual image
		 */
		$current_image = apply_filters( 'picu_single_image_data', $current_image, $post );

		$image_collection[] = array(
			'number' => $current_image['number'],
			'imageID' => $current_image['imageID'],
			'title' => $current_image['title'],
			'description' => $current_image['description'],
			'imagePath' => $current_image['imagePath'],
			'imagePath_small' => $current_image['imagePath_small'],
			'imagePath_original' => $current_image['imagePath_original'],
			'imagePath_srcset' => $current_image['imagePath_srcset'],
			'imagePath_small_srcset' => $current_image['imagePath_small_srcset'],
			'size' => $current_image['size'],
			'sizeLightbox' => $current_image['size_lightbox'],
			'orientation' => $current_image['orientation'],
			'selected' => $current_image['selected'],
			'markers' => $current_image['markers']
		);

		$imgnum++;
	}

	return $image_collection;

}



/**
 * Return JSON formated image collection
 *
 * @since 1.5.0
 *
 * @param $post – the picu collection object
 * @return string javascript object, containing the image collection
 *
 */

function picu_get_images( $post = '' ) {

	if ( empty( $post ) ) {
		$post = get_post();
	}

	// Get image IDs
	$include = get_post_meta( $post->ID, '_picu_collection_gallery_ids', true );
	$delivery_images = get_post_meta( $post->ID, '_picu_collection_delivery_ids', true );

	if ( ( 'delivered' == $post->post_status OR 'delivery-draft' == $post->post_status ) AND ! empty( $delivery_images ) ) {
		return json_encode( picu_get_image_collection( $delivery_images ), JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT );
	}
	elseif ( ! empty( $include ) AND 'delivered' != $post->post_status AND 'delivery-draft' != $post->post_status ) {
		return json_encode( picu_get_image_collection( $include ), JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT );
	}
	else {
		return '[ ]';
	}
}



/**
 * Helper function to add slashes and encode comments
 */
function picu_encode_marker_comment( &$item, $key ) {
	if ( $key == 'comment' ) {
		$item = addslashes( htmlspecialchars( $item, ENT_QUOTES, 'UTF-8' ) );
		$item = str_replace( '&amp;', '&', $item );
	}
}



/**
 * Create AppState JSON object
 *
 * @since 0.6.0
 *
 * @param $id, collection id
 * @return string, json
 *
 */

function picu_get_app_state() {

	$post = get_post();
	$id = $post->ID;

	$description = addslashes( wpautop( htmlspecialchars( get_post_meta( $id, '_picu_collection_description', true ), ENT_QUOTES, 'UTF-8' ) ) );

	$state = array(
		'version' => PICU_VERSION,
		'nonce' => wp_create_nonce( 'picu-ajax-security' ),
		'postid' => get_the_ID( $id ),
		'poststatus' => get_post_status( $id ),
		'title' => get_the_title( $id ),
		'date' => get_the_date( get_option( 'date_format' ), $id ),
		'description' => preg_replace( "/\r|\n|\t/", "", $description ),
		'ajaxurl' => admin_url( 'admin-ajax.php' ),
		'error_msg_no_imgs' => __( '<h2>No images found</h2><p>It seems there are no images in this collection.</p>', 'picu' ),
		'error_msg_filter_selected' => __( 'You have not selected any images.', 'picu' ),
		'error_msg_filter_unselected' => __( 'You have no <em>unselected</em> images.', 'picu' ),
		'reset_filter_msg' => __( 'Reset filter to show all images', 'picu' ),
		'select_at_least_one_image_msg' => __( 'You have to select at least one image.', 'picu' ),
		'already_approved_msg' => __( 'This collection has already been approved.', 'picu' ),
		'request_failed_error' => __( 'Error: Request failed.<br />Do you have a working internet connection?', 'picu' ),
	);

	// Add identifier
	$hashes = get_post_meta( get_the_ID(), '_picu_collection_hashes', true );

	// TODO: Also account for cookie
	if ( ! empty( $hashes ) AND ! empty( $_GET['ident'] ) ) {
		$hashes = get_post_meta( get_the_ID(), '_picu_collection_hashes', true );

		if ( array_key_exists( $_GET['ident'], $hashes ) ) {
			$state['ident'] = $_GET['ident'];

			if ( $hashes[$_GET['ident']]['status'] == 'approved' ) {
				$state['poststatus'] = 'approved';
			}
		}
	}

	/**
	 * Add filter to extend on app state parameters
	 */
	$state = apply_filters( 'picu_app_state', $state );

	$app_state = json_encode( $state, JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT );

	return $app_state;

}


/**
 * Collection list shortcode
 *
 * @since 1.2.0
 *
 * @param array $atts Shortcode attributes
 * @param string $content Content between the shortcode tags; displayed when no collections are found
 *
 * @return string HTML output of the collection list
 *
 */

function picu_list_collections( $atts, $content = null ) {

	// Setup default arguments
	$args = array(
		'post_type' => 'picu_collection',
		'orderby' => 'date',
		'posts_per_page' => -1
	);

	// TODO: Remove at sometime in the future, bc it is deprecated, replaced by `status` in version 1.5.9
	if ( is_array( $atts ) AND in_array( 'show_approved', $atts ) ) {
		$args['post_status'] = array( 'approved', 'sent' );
	}
	else {
		$args['post_status'] = 'sent';
	}

	// Check for status
	if ( is_array( $atts ) AND array_key_exists( 'status', $atts ) ) {
		$status = explode( ',', $atts['status'] );
		// Only allow certain predefine status: sent, approved, delivered
		$status = array_filter( $status, function( $item ) {
			if ( in_array( trim( $item ), [ 'sent', 'approved', 'delivered' ] ) ) {
				return true;
			}
			else {
				return false;
			}
		} );
		$args['post_status'] = $status;
	}

	// Check for IDs
	if ( isset( $atts['ids'] ) and ! empty( $atts['ids'] ) ) {
		$ids = explode( ',', $atts['ids'] );
		$ids = array_map( 'trim', $ids );
		$args['post__in'] = $ids;
	}

	// Check for email address
	if ( isset( $atts['email'] ) and ! empty( $atts['email'] ) ) {
		$email = sanitize_email( $atts['email'] );
	}

	// Check for current user
	if ( is_array( $atts ) AND in_array( 'current_user', $atts ) ) {
		$current_user = wp_get_current_user();
		if ( ! empty( $current_user->user_email ) ) {
			$email = $current_user->user_email;
		}
	}

	// Fill meta query arg for email
	if ( ! empty( $email ) ) {
		$args['meta_query'] = array(
			'relation' => 'OR',
			array(
				'key' => '_picu_collection_email_address',
				'value' => $email,
				'compare' => 'LIKE'
			),
			array(
				'key' => '_picu_delivery_email_address',
				'value' => $email,
				'compare' => 'LIKE'
			),
		);
	}

	// TODO: Extend query - show/hide multi-client collections

	// Filter query args
	$args = apply_filters( 'picu_list_collections_args', $args );

	$query = new WP_Query( $args );

	if ( $query->have_posts() ) {

		$collection_list = '<ul class="picu-collection-list">';

		while ( $query->have_posts() ) {
			$query->the_post();
			$collection_list .= '<li class="picu-status-' . get_post_status() . '"><a href="' . get_permalink() . '">' . get_the_title() . '</a></li>';
		}

		$collection_list .= '</ul>';

	}
	else {
		$collection_list = '<div class="picu-no-collections">' . $content . '</div>';
	}

	wp_reset_query();

	// Filter the collection list output
	$collection_list = apply_filters( 'picu_collection_list', $collection_list, $query, $atts, $content );

	return $collection_list;
}

add_shortcode( 'picu_list_collections', 'picu_list_collections' );


/**
 * Change the password form for picu collections
 *
 * @since 1.4.5
 *
 * @param $output
 * @return html
 *
 */
function picu_password_form( $output ) {

	global $post;

	if ( $post->post_type == 'picu_collection' ) {
		$label = 'pwbox-' . ( empty( $post->ID ) ? rand() : $post->ID );
		ob_start();
	?>
	<form action="<?php echo esc_url( site_url( 'wp-login.php?action=postpass', 'login_post' ) ); ?>" method="post">
		<p><label for="<?php echo $label; ?>"><?php _e( 'To view this collection, enter the password below.', 'picu' ); ?></label> <input name="post_password" id="<?php echo $label; ?>" type="password" size="20" maxlength="20" /></p>
		<p><input class="picu-button primary" type="submit" name="Submit" value="<?php echo esc_attr__( 'Enter', 'picu' ); ?>" /></p>
	</form>
	<?php
		$output = ob_get_clean();
	}

	return $output;
}

add_filter( 'the_password_form', 'picu_password_form' );


/**
 * Get the collection status
 * 
 * @since 1.6.0
 * 
 * @param int $post_id The collection/post id
 * @param string $ident The ident parameter, if it is a multi-client collection
 * 
 * @return string The collection status
 */
function picu_get_collection_status( $post_id, $ident = '' ) {

	if ( ! empty( $ident ) ) {
		$hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );
		if ( ! empty( $hashes[$ident]['status'] ) ) {
			return $hashes[$ident]['status'];
		}
	}

	return get_post_status( $post_id );
}


/**
 * Check if a collection is a multi-client collection
 *
 * @since 1.6.0
 *
 * @param int $post_id
 * @return bool
 *
 */
function picu_is_multi_collection( $post_id ) {

	$hashes = get_post_meta( $post_id, '_picu_collection_hashes', true );

	if ( ! empty( $hashes ) AND count( $hashes ) > 0 ) {
		return true;
	}
	
	return false;
}


/**
 * Only allow access to a multi-client collection if the identifier is present
 *
 * @since 1.6.0
 *
 */
function picu_collection_bouncer() {

	// Allow entry if collection is delivery-draft or delivered
	$post_status = get_post_status( get_the_ID() );
	if ( $post_status == 'delivery-draft' OR $post_status == 'delivered' ) {
		return true;
	}

	$hashes = get_post_meta( get_the_ID(), '_picu_collection_hashes', true );

	// If hashes is empty aka no multi client, allow access
	if ( empty( $hashes ) ) {
		return true;
	}

	// If the indentifier is valid, allow access
	if ( ! empty( $_GET['ident'] ) AND array_key_exists( $_GET['ident'], $hashes ) ) {
		return true;
	}
	else {

		global $post;

		// If it is the post author, display a list of identities
		if ( $post->post_author == get_current_user_id() ) {
			$content = '<p>⚠️ ' . __( 'To view this multi-client collection, you need to identify as one of the recipients:', 'picu' ) . '</p>';
			$content .= '<ul>';
			foreach( $hashes as $key => $hash ) {
				$content .= '<li><a href="' . esc_url( add_query_arg( 'ident', $key, get_the_permalink() ) ) .'">' . $hash['email'] . '</a></li>';
			}
			$content .= '</ul>';
			$content .= '<p>(' . __( 'This is only available to you, the collection author.', 'picu' ) . ')</p>';
			wp_die( $content );
		}
		// Show 404 for everyone else
		else {
			status_header( 404 );
			nocache_headers();
			include( get_404_template() );
			die();
		}
	}
}